@@ -13,6 +13,7 @@
 #include <string>
 
 #include "db/db_test_util.h"
+#include "db/write_stall_stats.h"
 #include "options/cf_options.h"
 #include "port/stack_trace.h"
 #include "rocksdb/listener.h"
@@ -55,7 +56,6 @@ class DBPropertiesTest : public DBTestBase {
   }
 };
 
-#ifndef ROCKSDB_LITE
 TEST_F(DBPropertiesTest, Empty) {
   do {
     Options options;
@@ -270,7 +270,8 @@ void GetExpectedTableProperties(
   const int kDeletionCount = kTableCount * kDeletionsPerTable;
   const int kMergeCount = kTableCount * kMergeOperandsPerTable;
   const int kRangeDeletionCount = kTableCount * kRangeDeletionsPerTable;
-  const int kKeyCount = kPutCount + kDeletionCount + kMergeCount + kRangeDeletionCount;
+  const int kKeyCount =
+      kPutCount + kDeletionCount + kMergeCount + kRangeDeletionCount;
   const int kAvgSuccessorSize = kKeySize / 5;
   const int kEncodingSavePerKey = kKeySize / 4;
   expected_tp->raw_key_size = kKeyCount * (kKeySize + 8);
@@ -281,7 +282,8 @@ void GetExpectedTableProperties(
   expected_tp->num_merge_operands = kMergeCount;
   expected_tp->num_range_deletions = kRangeDeletionCount;
   expected_tp->num_data_blocks =
-      kTableCount * (kKeysPerTable * (kKeySize - kEncodingSavePerKey + kValueSize)) /
+      kTableCount *
+      (kKeysPerTable * (kKeySize - kEncodingSavePerKey + kValueSize)) /
       kBlockSize;
   expected_tp->data_size =
       kTableCount * (kKeysPerTable * (kKeySize + 8 + kValueSize));
@@ -1110,7 +1112,6 @@ TEST_F(DBPropertiesTest, EstimateCompressionRatio) {
   ASSERT_GT(CompressionRatioAtLevel(1), 10.0);
 }
 
-#endif  // ROCKSDB_LITE
 
 class CountingUserTblPropCollector : public TablePropertiesCollector {
  public:
@@ -1120,7 +1121,8 @@ class CountingUserTblPropCollector : public TablePropertiesCollector {
     std::string encoded;
     PutVarint32(&encoded, count_);
     *properties = UserCollectedProperties{
-        {"CountingUserTblPropCollector", message_}, {"Count", encoded},
+        {"CountingUserTblPropCollector", message_},
+        {"Count", encoded},
     };
     return Status::OK();
   }
@@ -1260,7 +1262,6 @@ class BlockCountingTablePropertiesCollectorFactory
   }
 };
 
-#ifndef ROCKSDB_LITE
 TEST_F(DBPropertiesTest, GetUserDefinedTableProperties) {
   Options options = CurrentOptions();
   options.level0_file_num_compaction_trigger = (1 << 30);
@@ -1300,7 +1301,6 @@ TEST_F(DBPropertiesTest, GetUserDefinedTableProperties) {
   ASSERT_OK(dbfull()->TEST_CompactRange(0, nullptr, nullptr));
   ASSERT_GT(collector_factory->num_created_, 0U);
 }
-#endif  // ROCKSDB_LITE
 
 TEST_F(DBPropertiesTest, UserDefinedTablePropertiesContext) {
   Options options = CurrentOptions();
@@ -1362,7 +1362,6 @@ TEST_F(DBPropertiesTest, UserDefinedTablePropertiesContext) {
   ASSERT_GT(collector_factory->num_created_, 0U);
 }
 
-#ifndef ROCKSDB_LITE
 TEST_F(DBPropertiesTest, TablePropertiesNeedCompactTest) {
   Random rnd(301);
 
@@ -1845,8 +1844,8 @@ TEST_F(DBPropertiesTest, BlobCacheProperties) {
 
   // Insert unpinned blob to the cache and check size.
   constexpr size_t kSize1 = 70;
-  ASSERT_OK(blob_cache->Insert("blob1", nullptr /*value*/, kSize1,
-                               nullptr /*deleter*/));
+  ASSERT_OK(blob_cache->Insert("blob1", nullptr /*value*/,
+                               &kNoopCacheItemHelper, kSize1));
   ASSERT_TRUE(db_->GetIntProperty(DB::Properties::kBlobCacheCapacity, &value));
   ASSERT_EQ(kCapacity, value);
   ASSERT_TRUE(db_->GetIntProperty(DB::Properties::kBlobCacheUsage, &value));
@@ -1858,8 +1857,8 @@ TEST_F(DBPropertiesTest, BlobCacheProperties) {
   // Insert pinned blob to the cache and check size.
   constexpr size_t kSize2 = 60;
   Cache::Handle* blob2 = nullptr;
-  ASSERT_OK(blob_cache->Insert("blob2", nullptr /*value*/, kSize2,
-                               nullptr /*deleter*/, &blob2));
+  ASSERT_OK(blob_cache->Insert("blob2", nullptr /*value*/,
+                               &kNoopCacheItemHelper, kSize2, &blob2));
   ASSERT_NE(nullptr, blob2);
   ASSERT_TRUE(db_->GetIntProperty(DB::Properties::kBlobCacheCapacity, &value));
   ASSERT_EQ(kCapacity, value);
@@ -1873,8 +1872,8 @@ TEST_F(DBPropertiesTest, BlobCacheProperties) {
   // Insert another pinned blob to make the cache over-sized.
   constexpr size_t kSize3 = 80;
   Cache::Handle* blob3 = nullptr;
-  ASSERT_OK(blob_cache->Insert("blob3", nullptr /*value*/, kSize3,
-                               nullptr /*deleter*/, &blob3));
+  ASSERT_OK(blob_cache->Insert("blob3", nullptr /*value*/,
+                               &kNoopCacheItemHelper, kSize3, &blob3));
   ASSERT_NE(nullptr, blob3);
   ASSERT_TRUE(db_->GetIntProperty(DB::Properties::kBlobCacheCapacity, &value));
   ASSERT_EQ(kCapacity, value);
@@ -1953,8 +1952,8 @@ TEST_F(DBPropertiesTest, BlockCacheProperties) {
 
   // Insert unpinned item to the cache and check size.
   constexpr size_t kSize1 = 50;
-  ASSERT_OK(block_cache->Insert("item1", nullptr /*value*/, kSize1,
-                                nullptr /*deleter*/));
+  ASSERT_OK(block_cache->Insert("item1", nullptr /*value*/,
+                                &kNoopCacheItemHelper, kSize1));
   ASSERT_TRUE(db_->GetIntProperty(DB::Properties::kBlockCacheCapacity, &value));
   ASSERT_EQ(kCapacity, value);
   ASSERT_TRUE(db_->GetIntProperty(DB::Properties::kBlockCacheUsage, &value));
@@ -1966,8 +1965,8 @@ TEST_F(DBPropertiesTest, BlockCacheProperties) {
   // Insert pinned item to the cache and check size.
   constexpr size_t kSize2 = 30;
   Cache::Handle* item2 = nullptr;
-  ASSERT_OK(block_cache->Insert("item2", nullptr /*value*/, kSize2,
-                                nullptr /*deleter*/, &item2));
+  ASSERT_OK(block_cache->Insert("item2", nullptr /*value*/,
+                                &kNoopCacheItemHelper, kSize2, &item2));
   ASSERT_NE(nullptr, item2);
   ASSERT_TRUE(db_->GetIntProperty(DB::Properties::kBlockCacheCapacity, &value));
   ASSERT_EQ(kCapacity, value);
@@ -1980,8 +1979,8 @@ TEST_F(DBPropertiesTest, BlockCacheProperties) {
   // Insert another pinned item to make the cache over-sized.
   constexpr size_t kSize3 = 80;
   Cache::Handle* item3 = nullptr;
-  ASSERT_OK(block_cache->Insert("item3", nullptr /*value*/, kSize3,
-                                nullptr /*deleter*/, &item3));
+  ASSERT_OK(block_cache->Insert("item3", nullptr /*value*/,
+                                &kNoopCacheItemHelper, kSize3, &item3));
   ASSERT_NE(nullptr, item2);
   ASSERT_TRUE(db_->GetIntProperty(DB::Properties::kBlockCacheCapacity, &value));
   ASSERT_EQ(kCapacity, value);
@@ -2108,6 +2107,181 @@ TEST_F(DBPropertiesTest, GetMapPropertyBlockCacheEntryStats) {
   ASSERT_EQ(3 * kNumCacheEntryRoles + 4, values.size());
 }
 
+TEST_F(DBPropertiesTest, WriteStallStatsSanityCheck) {
+  for (uint32_t i = 0; i < static_cast<uint32_t>(WriteStallCause::kNone); ++i) {
+    WriteStallCause cause = static_cast<WriteStallCause>(i);
+    const std::string& str = WriteStallCauseToHyphenString(cause);
+    ASSERT_TRUE(!str.empty())
+        << "Please ensure mapping from `WriteStallCause` to "
+           "`WriteStallCauseToHyphenString` is complete";
+    if (cause == WriteStallCause::kCFScopeWriteStallCauseEnumMax ||
+        cause == WriteStallCause::kDBScopeWriteStallCauseEnumMax) {
+      ASSERT_EQ(str, InvalidWriteStallHyphenString())
+          << "Please ensure order in `WriteStallCauseToHyphenString` is "
+             "consistent with `WriteStallCause`";
+    }
+  }
+
+  for (uint32_t i = 0; i < static_cast<uint32_t>(WriteStallCondition::kNormal);
+       ++i) {
+    WriteStallCondition condition = static_cast<WriteStallCondition>(i);
+    const std::string& str = WriteStallConditionToHyphenString(condition);
+    ASSERT_TRUE(!str.empty())
+        << "Please ensure mapping from `WriteStallCondition` to "
+           "`WriteStallConditionToHyphenString` is complete";
+  }
+
+  for (uint32_t i = 0; i < static_cast<uint32_t>(WriteStallCause::kNone); ++i) {
+    for (uint32_t j = 0;
+         j < static_cast<uint32_t>(WriteStallCondition::kNormal); ++j) {
+      WriteStallCause cause = static_cast<WriteStallCause>(i);
+      WriteStallCondition condition = static_cast<WriteStallCondition>(j);
+
+      if (isCFScopeWriteStallCause(cause)) {
+        ASSERT_TRUE(InternalCFStat(cause, condition) !=
+                    InternalStats::INTERNAL_CF_STATS_ENUM_MAX)
+            << "Please ensure the combination of WriteStallCause(" +
+                   std::to_string(static_cast<uint32_t>(cause)) +
+                   ") + WriteStallCondition(" +
+                   std::to_string(static_cast<uint32_t>(condition)) +
+                   ") is correctly mapped to a valid `InternalStats` or bypass "
+                   "its check in this test";
+      } else if (isDBScopeWriteStallCause(cause)) {
+        InternalStats::InternalDBStatsType internal_db_stat =
+            InternalDBStat(cause, condition);
+        if (internal_db_stat == InternalStats::kIntStatsNumMax) {
+          ASSERT_TRUE(cause == WriteStallCause::kWriteBufferManagerLimit &&
+                      condition == WriteStallCondition::kDelayed)
+              << "Please ensure the combination of WriteStallCause(" +
+                     std::to_string(static_cast<uint32_t>(cause)) +
+                     ") + WriteStallCondition(" +
+                     std::to_string(static_cast<uint32_t>(condition)) +
+                     ") is correctly mapped to a valid `InternalStats` or "
+                     "bypass its check in this test";
+        }
+      } else if (cause != WriteStallCause::kCFScopeWriteStallCauseEnumMax &&
+                 cause != WriteStallCause::kDBScopeWriteStallCauseEnumMax) {
+        ASSERT_TRUE(false) << "Please ensure the WriteStallCause(" +
+                                  std::to_string(static_cast<uint32_t>(cause)) +
+                                  ") is either CF-scope or DB-scope write "
+                                  "stall cause in enum `WriteStallCause`";
+      }
+    }
+  }
+}
+TEST_F(DBPropertiesTest, GetMapPropertyWriteStallStats) {
+  Options options = CurrentOptions();
+  CreateAndReopenWithCF({"heavy_write_cf"}, options);
+
+  for (auto test_cause : {WriteStallCause::kWriteBufferManagerLimit,
+                          WriteStallCause::kMemtableLimit}) {
+    if (test_cause == WriteStallCause::kWriteBufferManagerLimit) {
+      options.write_buffer_manager.reset(
+          new WriteBufferManager(100000, nullptr, true));
+    } else if (test_cause == WriteStallCause::kMemtableLimit) {
+      options.max_write_buffer_number = 2;
+      options.disable_auto_compactions = true;
+    }
+    ReopenWithColumnFamilies({"default", "heavy_write_cf"}, options);
+
+    // Assert initial write stall stats are all 0
+    std::map<std::string, std::string> db_values;
+    ASSERT_TRUE(dbfull()->GetMapProperty(DB::Properties::kDBWriteStallStats,
+                                         &db_values));
+    ASSERT_EQ(std::stoi(db_values[WriteStallStatsMapKeys::CauseConditionCount(
+                  WriteStallCause::kWriteBufferManagerLimit,
+                  WriteStallCondition::kStopped)]),
+              0);
+
+    for (int cf = 0; cf <= 1; ++cf) {
+      std::map<std::string, std::string> cf_values;
+      ASSERT_TRUE(dbfull()->GetMapProperty(
+          handles_[cf], DB::Properties::kCFWriteStallStats, &cf_values));
+      ASSERT_EQ(std::stoi(cf_values[WriteStallStatsMapKeys::TotalStops()]), 0);
+      ASSERT_EQ(std::stoi(cf_values[WriteStallStatsMapKeys::TotalDelays()]), 0);
+    }
+
+    // Pause flush thread to help coerce write stall
+    std::unique_ptr<test::SleepingBackgroundTask> sleeping_task(
+        new test::SleepingBackgroundTask());
+    env_->SetBackgroundThreads(1, Env::HIGH);
+    env_->Schedule(&test::SleepingBackgroundTask::DoSleepTask,
+                   sleeping_task.get(), Env::Priority::HIGH);
+    sleeping_task->WaitUntilSleeping();
+
+    // Coerce write stall
+    if (test_cause == WriteStallCause::kWriteBufferManagerLimit) {
+      ASSERT_OK(dbfull()->Put(
+          WriteOptions(), handles_[1], Key(1),
+          DummyString(options.write_buffer_manager->buffer_size())));
+
+      WriteOptions wo;
+      wo.no_slowdown = true;
+      Status s = dbfull()->Put(
+          wo, handles_[1], Key(2),
+          DummyString(options.write_buffer_manager->buffer_size()));
+      ASSERT_TRUE(s.IsIncomplete());
+      ASSERT_TRUE(s.ToString().find("Write stall") != std::string::npos);
+    } else if (test_cause == WriteStallCause::kMemtableLimit) {
+      FlushOptions fo;
+      fo.allow_write_stall = true;
+      fo.wait = false;
+
+      ASSERT_OK(
+          dbfull()->Put(WriteOptions(), handles_[1], Key(1), DummyString(1)));
+      ASSERT_OK(dbfull()->Flush(fo, handles_[1]));
+
+      ASSERT_OK(
+          dbfull()->Put(WriteOptions(), handles_[1], Key(2), DummyString(1)));
+      ASSERT_OK(dbfull()->Flush(fo, handles_[1]));
+    }
+
+    if (test_cause == WriteStallCause::kWriteBufferManagerLimit) {
+      db_values.clear();
+      EXPECT_TRUE(dbfull()->GetMapProperty(DB::Properties::kDBWriteStallStats,
+                                           &db_values));
+      EXPECT_EQ(std::stoi(db_values[WriteStallStatsMapKeys::CauseConditionCount(
+                    WriteStallCause::kWriteBufferManagerLimit,
+                    WriteStallCondition::kStopped)]),
+                1);
+      // `WriteStallCause::kWriteBufferManagerLimit` should not result in any
+      // CF-scope write stall stats changes
+      for (int cf = 0; cf <= 1; ++cf) {
+        std::map<std::string, std::string> cf_values;
+        EXPECT_TRUE(dbfull()->GetMapProperty(
+            handles_[cf], DB::Properties::kCFWriteStallStats, &cf_values));
+        EXPECT_EQ(std::stoi(cf_values[WriteStallStatsMapKeys::TotalStops()]),
+                  0);
+        EXPECT_EQ(std::stoi(cf_values[WriteStallStatsMapKeys::TotalDelays()]),
+                  0);
+      }
+    } else if (test_cause == WriteStallCause::kMemtableLimit) {
+      for (int cf = 0; cf <= 1; ++cf) {
+        std::map<std::string, std::string> cf_values;
+        EXPECT_TRUE(dbfull()->GetMapProperty(
+            handles_[cf], DB::Properties::kCFWriteStallStats, &cf_values));
+        EXPECT_EQ(std::stoi(cf_values[WriteStallStatsMapKeys::TotalStops()]),
+                  cf == 1 ? 1 : 0);
+        EXPECT_EQ(
+            std::stoi(cf_values[WriteStallStatsMapKeys::CauseConditionCount(
+                WriteStallCause::kMemtableLimit,
+                WriteStallCondition::kStopped)]),
+            cf == 1 ? 1 : 0);
+        EXPECT_EQ(std::stoi(cf_values[WriteStallStatsMapKeys::TotalDelays()]),
+                  0);
+        EXPECT_EQ(
+            std::stoi(cf_values[WriteStallStatsMapKeys::CauseConditionCount(
+                WriteStallCause::kMemtableLimit,
+                WriteStallCondition::kDelayed)]),
+            0);
+      }
+    }
+
+    sleeping_task->WakeUp();
+    sleeping_task->WaitUntilDone();
+  }
+}
+
 namespace {
 std::string PopMetaIndexKey(InternalIterator* meta_iter) {
   Status s = meta_iter->status();
@@ -2122,7 +2296,7 @@ std::string PopMetaIndexKey(InternalIterator* meta_iter) {
   }
 }
 
-}  // namespace
+}  // anonymous namespace
 
 TEST_F(DBPropertiesTest, TableMetaIndexKeys) {
   // This is to detect unexpected churn in metaindex block keys. This is more
@@ -2159,8 +2333,9 @@ TEST_F(DBPropertiesTest, TableMetaIndexKeys) {
 
     // Read metaindex
     BlockContents bc;
-    ASSERT_OK(ReadMetaIndexBlockInFile(r.get(), file_size, 0U,
-                                       ImmutableOptions(options), &bc));
+    const ReadOptions read_options;
+    ASSERT_OK(ReadMetaIndexBlockInFile(
+        r.get(), file_size, 0U, ImmutableOptions(options), read_options, &bc));
     Block metaindex_block(std::move(bc));
     std::unique_ptr<InternalIterator> meta_iter;
     meta_iter.reset(metaindex_block.NewMetaIterator());
@@ -2192,7 +2367,6 @@ TEST_F(DBPropertiesTest, TableMetaIndexKeys) {
   } while (ChangeOptions());
 }
 
-#endif  // ROCKSDB_LITE
 
 }  // namespace ROCKSDB_NAMESPACE
 
