@@ -23,6 +23,7 @@
 #include "cache/cache_entry_stats.h"
 #include "db/column_family.h"
 #include "db/db_impl/db_impl.h"
+#include "db/write_stall_stats.h"
 #include "port/port.h"
 #include "rocksdb/system_clock.h"
 #include "rocksdb/table.h"
@@ -32,7 +33,6 @@
 
 namespace ROCKSDB_NAMESPACE {
 
-#ifndef ROCKSDB_LITE
 
 const std::map<LevelStatType, LevelStat> InternalStats::compaction_level_stats =
     {
@@ -78,6 +78,10 @@ const std::map<InternalStats::InternalDBStatsType, DBStatInfo>
          DBStatInfo{"db.user_writes_with_wal"}},
         {InternalStats::kIntStatsWriteStallMicros,
          DBStatInfo{"db.user_write_stall_micros"}},
+        {InternalStats::kIntStatsWriteBufferManagerLimitStopsCounts,
+         DBStatInfo{WriteStallStatsMapKeys::CauseConditionCount(
+             WriteStallCause::kWriteBufferManagerLimit,
+             WriteStallCondition::kStopped)}},
 };
 
 namespace {
@@ -244,7 +248,9 @@ static const std::string cfstats = "cfstats";
 static const std::string cfstats_no_file_histogram =
     "cfstats-no-file-histogram";
 static const std::string cf_file_histogram = "cf-file-histogram";
+static const std::string cf_write_stall_stats = "cf-write-stall-stats";
 static const std::string dbstats = "dbstats";
+static const std::string db_write_stall_stats = "db-write-stall-stats";
 static const std::string levelstats = "levelstats";
 static const std::string block_cache_entry_stats = "block-cache-entry-stats";
 static const std::string fast_block_cache_entry_stats =
@@ -324,6 +330,10 @@ const std::string DB::Properties::kCFStatsNoFileHistogram =
     rocksdb_prefix + cfstats_no_file_histogram;
 const std::string DB::Properties::kCFFileHistogram =
     rocksdb_prefix + cf_file_histogram;
+const std::string DB::Properties::kCFWriteStallStats =
+    rocksdb_prefix + cf_write_stall_stats;
+const std::string DB::Properties::kDBWriteStallStats =
+    rocksdb_prefix + db_write_stall_stats;
 const std::string DB::Properties::kDBStats = rocksdb_prefix + dbstats;
 const std::string DB::Properties::kLevelStats = rocksdb_prefix + levelstats;
 const std::string DB::Properties::kBlockCacheEntryStats =
@@ -423,6 +433,10 @@ const std::string DB::Properties::kBlobCacheUsage =
 const std::string DB::Properties::kBlobCachePinnedUsage =
     rocksdb_prefix + blob_cache_pinned_usage;
 
+const std::string InternalStats::kPeriodicCFStats =
+    DB::Properties::kCFStats + ".periodic";
+const int InternalStats::kMaxNoChangePeriodSinceDump = 8;
+
 const UnorderedMap<std::string, DBPropertyInfo>
     InternalStats::ppt_name_to_info = {
         {DB::Properties::kNumFilesAtLevelPrefix,
@@ -438,15 +452,24 @@ const UnorderedMap<std::string, DBPropertyInfo>
         {DB::Properties::kCFStats,
          {false, &InternalStats::HandleCFStats, nullptr,
           &InternalStats::HandleCFMapStats, nullptr}},
+        {InternalStats::kPeriodicCFStats,
+         {false, &InternalStats::HandleCFStatsPeriodic, nullptr, nullptr,
+          nullptr}},
         {DB::Properties::kCFStatsNoFileHistogram,
          {false, &InternalStats::HandleCFStatsNoFileHistogram, nullptr, nullptr,
           nullptr}},
         {DB::Properties::kCFFileHistogram,
          {false, &InternalStats::HandleCFFileHistogram, nullptr, nullptr,
           nullptr}},
+        {DB::Properties::kCFWriteStallStats,
+         {false, &InternalStats::HandleCFWriteStallStats, nullptr,
+          &InternalStats::HandleCFWriteStallStatsMap, nullptr}},
         {DB::Properties::kDBStats,
          {false, &InternalStats::HandleDBStats, nullptr,
           &InternalStats::HandleDBMapStats, nullptr}},
+        {DB::Properties::kDBWriteStallStats,
+         {false, &InternalStats::HandleDBWriteStallStats, nullptr,
+          &InternalStats::HandleDBWriteStallStatsMap, nullptr}},
         {DB::Properties::kBlockCacheEntryStats,
          {true, &InternalStats::HandleBlockCacheEntryStats, nullptr,
           &InternalStats::HandleBlockCacheEntryStatsMap, nullptr}},
@@ -605,6 +628,7 @@ InternalStats::InternalStats(int num_levels, SystemClock* clock,
       comp_stats_(num_levels),
       comp_stats_by_pri_(Env::Priority::TOTAL),
       file_read_latency_(num_levels),
+      has_cf_change_since_dump_(true),
       bg_error_count_(0),
       number_levels_(num_levels),
       clock_(clock),
@@ -651,17 +675,13 @@ void InternalStats::CollectCacheEntryStats(bool foreground) {
                                              min_interval_factor);
 }
 
-std::function<void(const Slice&, void*, size_t, Cache::DeleterFn)>
+std::function<void(const Slice& key, Cache::ObjectPtr value, size_t charge,
+                   const Cache::CacheItemHelper* helper)>
 InternalStats::CacheEntryRoleStats::GetEntryCallback() {
-  return [&](const Slice& /*key*/, void* /*value*/, size_t charge,
-             Cache::DeleterFn deleter) {
-    auto e = role_map_.find(deleter);
-    size_t role_idx;
-    if (e == role_map_.end()) {
-      role_idx = static_cast<size_t>(CacheEntryRole::kMisc);
-    } else {
-      role_idx = static_cast<size_t>(e->second);
-    }
+  return [&](const Slice& /*key*/, Cache::ObjectPtr /*value*/, size_t charge,
+             const Cache::CacheItemHelper* helper) -> void {
+    size_t role_idx =
+        static_cast<size_t>(helper ? helper->role : CacheEntryRole::kMisc);
     entry_counts[role_idx]++;
     total_charges[role_idx] += charge;
   };
@@ -672,7 +692,6 @@ void InternalStats::CacheEntryRoleStats::BeginCollection(
   Clear();
   last_start_time_micros_ = start_time_micros;
   ++collection_count;
-  role_map_ = CopyCacheDeleterRoleMap();
   std::ostringstream str;
   str << cache->Name() << "@" << static_cast<void*>(cache) << "#"
       << port::GetProcessID();
@@ -681,6 +700,7 @@ void InternalStats::CacheEntryRoleStats::BeginCollection(
   cache_usage = cache->GetUsage();
   table_size = cache->GetTableAddressCount();
   occupancy = cache->GetOccupancyCount();
+  hash_seed = cache->GetHashSeed();
 }
 
 void InternalStats::CacheEntryRoleStats::EndCollection(
@@ -705,7 +725,7 @@ std::string InternalStats::CacheEntryRoleStats::ToString(
   std::ostringstream str;
   str << "Block cache " << cache_id
       << " capacity: " << BytesToHumanString(cache_capacity)
-      << " usage: " << BytesToHumanString(cache_usage)
+      << " seed: " << hash_seed << " usage: " << BytesToHumanString(cache_usage)
       << " table_size: " << table_size << " occupancy: " << occupancy
       << " collections: " << collection_count
       << " last_copies: " << copies_of_last_collection
@@ -1041,9 +1061,41 @@ bool InternalStats::HandleCFStats(std::string* value, Slice /*suffix*/) {
   return true;
 }
 
+bool InternalStats::HandleCFStatsPeriodic(std::string* value,
+                                          Slice /*suffix*/) {
+  bool has_change = has_cf_change_since_dump_;
+  if (!has_change) {
+    // If file histogram changes, there is activity in this period too.
+    uint64_t new_histogram_num = 0;
+    for (int level = 0; level < number_levels_; level++) {
+      new_histogram_num += file_read_latency_[level].num();
+    }
+    new_histogram_num += blob_file_read_latency_.num();
+    if (new_histogram_num != last_histogram_num) {
+      has_change = true;
+      last_histogram_num = new_histogram_num;
+    }
+  }
+  if (has_change) {
+    no_cf_change_period_since_dump_ = 0;
+    has_cf_change_since_dump_ = false;
+  } else if (no_cf_change_period_since_dump_++ > 0) {
+    // Not ready to sync
+    if (no_cf_change_period_since_dump_ == kMaxNoChangePeriodSinceDump) {
+      // Next periodic, we need to dump stats even if there is no change.
+      no_cf_change_period_since_dump_ = 0;
+    }
+    return true;
+  }
+
+  DumpCFStatsNoFileHistogram(/*is_periodic=*/true, value);
+  DumpCFFileHistogram(value);
+  return true;
+}
+
 bool InternalStats::HandleCFStatsNoFileHistogram(std::string* value,
                                                  Slice /*suffix*/) {
-  DumpCFStatsNoFileHistogram(value);
+  DumpCFStatsNoFileHistogram(/*is_periodic=*/false, value);
   return true;
 }
 
@@ -1053,6 +1105,18 @@ bool InternalStats::HandleCFFileHistogram(std::string* value,
   return true;
 }
 
+bool InternalStats::HandleCFWriteStallStats(std::string* value,
+                                            Slice /*suffix*/) {
+  DumpCFStatsWriteStall(value);
+  return true;
+}
+
+bool InternalStats::HandleCFWriteStallStatsMap(
+    std::map<std::string, std::string>* value, Slice /*suffix*/) {
+  DumpCFMapStatsWriteStall(value);
+  return true;
+}
+
 bool InternalStats::HandleDBMapStats(
     std::map<std::string, std::string>* db_stats, Slice /*suffix*/) {
   DumpDBMapStats(db_stats);
@@ -1064,6 +1128,18 @@ bool InternalStats::HandleDBStats(std::string* value, Slice /*suffix*/) {
   return true;
 }
 
+bool InternalStats::HandleDBWriteStallStats(std::string* value,
+                                            Slice /*suffix*/) {
+  DumpDBStatsWriteStall(value);
+  return true;
+}
+
+bool InternalStats::HandleDBWriteStallStatsMap(
+    std::map<std::string, std::string>* value, Slice /*suffix*/) {
+  DumpDBMapStatsWriteStall(value);
+  return true;
+}
+
 bool InternalStats::HandleSsTables(std::string* value, Slice /*suffix*/) {
   auto* current = cfd_->current();
   *value = current->DebugString(true, true);
@@ -1073,7 +1149,9 @@ bool InternalStats::HandleSsTables(std::string* value, Slice /*suffix*/) {
 bool InternalStats::HandleAggregatedTableProperties(std::string* value,
                                                     Slice /*suffix*/) {
   std::shared_ptr<const TableProperties> tp;
-  auto s = cfd_->current()->GetAggregatedTableProperties(&tp);
+  // TODO: plumb Env::IOActivity
+  const ReadOptions read_options;
+  auto s = cfd_->current()->GetAggregatedTableProperties(read_options, &tp);
   if (!s.ok()) {
     return false;
   }
@@ -1093,7 +1171,9 @@ static std::map<std::string, std::string> MapUint64ValuesToString(
 bool InternalStats::HandleAggregatedTablePropertiesMap(
     std::map<std::string, std::string>* values, Slice /*suffix*/) {
   std::shared_ptr<const TableProperties> tp;
-  auto s = cfd_->current()->GetAggregatedTableProperties(&tp);
+  // TODO: plumb Env::IOActivity
+  const ReadOptions read_options;
+  auto s = cfd_->current()->GetAggregatedTableProperties(read_options, &tp);
   if (!s.ok()) {
     return false;
   }
@@ -1109,8 +1189,10 @@ bool InternalStats::HandleAggregatedTablePropertiesAtLevel(std::string* values,
     return false;
   }
   std::shared_ptr<const TableProperties> tp;
+  // TODO: plumb Env::IOActivity
+  const ReadOptions read_options;
   auto s = cfd_->current()->GetAggregatedTableProperties(
-      &tp, static_cast<int>(level));
+      read_options, &tp, static_cast<int>(level));
   if (!s.ok()) {
     return false;
   }
@@ -1126,8 +1208,10 @@ bool InternalStats::HandleAggregatedTablePropertiesAtLevelMap(
     return false;
   }
   std::shared_ptr<const TableProperties> tp;
+  // TODO: plumb Env::IOActivity
+  const ReadOptions read_options;
   auto s = cfd_->current()->GetAggregatedTableProperties(
-      &tp, static_cast<int>(level));
+      read_options, &tp, static_cast<int>(level));
   if (!s.ok()) {
     return false;
   }
@@ -1322,7 +1406,11 @@ bool InternalStats::HandleEstimatePendingCompactionBytes(uint64_t* value,
 bool InternalStats::HandleEstimateTableReadersMem(uint64_t* value,
                                                   DBImpl* /*db*/,
                                                   Version* version) {
-  *value = (version == nullptr) ? 0 : version->GetMemoryUsageByTableReaders();
+  // TODO: plumb Env::IOActivity
+  const ReadOptions read_options;
+  *value = (version == nullptr)
+               ? 0
+               : version->GetMemoryUsageByTableReaders(read_options);
   return true;
 }
 
@@ -1373,9 +1461,10 @@ bool InternalStats::HandleEstimateOldestKeyTime(uint64_t* value, DBImpl* /*db*/,
           ->compaction_options_fifo.allow_compaction) {
     return false;
   }
-
+  // TODO: plumb Env::IOActivity
+  const ReadOptions read_options;
   TablePropertiesCollection collection;
-  auto s = cfd_->current()->GetPropertiesOfAllTables(&collection);
+  auto s = cfd_->current()->GetPropertiesOfAllTables(read_options, &collection);
   if (!s.ok()) {
     return false;
   }
@@ -1546,6 +1635,10 @@ void InternalStats::DumpDBStats(std::string* value) {
                10000.0 / std::max(interval_seconds_up, 0.001));
   value->append(buf);
 
+  std::string write_stall_stats;
+  DumpDBStatsWriteStall(&write_stall_stats);
+  value->append(write_stall_stats);
+
   db_stats_snapshot_.seconds_up = seconds_up;
   db_stats_snapshot_.ingest_bytes = user_bytes_written;
   db_stats_snapshot_.write_other = write_other;
@@ -1557,6 +1650,58 @@ void InternalStats::DumpDBStats(std::string* value) {
   db_stats_snapshot_.write_stall_micros = write_stall_micros;
 }
 
+void InternalStats::DumpDBMapStatsWriteStall(
+    std::map<std::string, std::string>* value) {
+  constexpr uint32_t max_db_scope_write_stall_cause =
+      static_cast<uint32_t>(WriteStallCause::kDBScopeWriteStallCauseEnumMax);
+
+  for (uint32_t i =
+           max_db_scope_write_stall_cause - kNumDBScopeWriteStallCauses;
+       i < max_db_scope_write_stall_cause; ++i) {
+    for (uint32_t j = 0;
+         j < static_cast<uint32_t>(WriteStallCondition::kNormal); ++j) {
+      WriteStallCause cause = static_cast<WriteStallCause>(i);
+      WriteStallCondition condition = static_cast<WriteStallCondition>(j);
+      InternalStats::InternalDBStatsType internal_db_stat =
+          InternalDBStat(cause, condition);
+
+      if (internal_db_stat == InternalStats::kIntStatsNumMax) {
+        continue;
+      }
+
+      std::string name =
+          WriteStallStatsMapKeys::CauseConditionCount(cause, condition);
+      uint64_t stat =
+          db_stats_[static_cast<std::size_t>(internal_db_stat)].load(
+              std::memory_order_relaxed);
+      (*value)[name] = std::to_string(stat);
+    }
+  }
+}
+
+void InternalStats::DumpDBStatsWriteStall(std::string* value) {
+  assert(value);
+
+  std::map<std::string, std::string> write_stall_stats_map;
+  DumpDBMapStatsWriteStall(&write_stall_stats_map);
+
+  std::ostringstream str;
+  str << "Write Stall (count): ";
+
+  for (auto write_stall_stats_map_iter = write_stall_stats_map.begin();
+       write_stall_stats_map_iter != write_stall_stats_map.end();
+       write_stall_stats_map_iter++) {
+    const auto& name_and_stat = *write_stall_stats_map_iter;
+    str << name_and_stat.first << ": " << name_and_stat.second;
+    if (std::next(write_stall_stats_map_iter) == write_stall_stats_map.end()) {
+      str << "\n";
+    } else {
+      str << ", ";
+    }
+  }
+  *value = str.str();
+}
+
 /**
  * Dump Compaction Level stats to a map of stat name with "compaction." prefix
  * to value in double as string. The level in stat name is represented with
@@ -1583,7 +1728,7 @@ void InternalStats::DumpCFMapStats(
     }
   }
 
-  DumpCFMapStatsIOStalls(cf_stats);
+  DumpCFMapStatsWriteStall(cf_stats);
 }
 
 void InternalStats::DumpCFMapStats(
@@ -1675,44 +1820,98 @@ void InternalStats::DumpCFMapStatsByPriority(
   }
 }
 
-void InternalStats::DumpCFMapStatsIOStalls(
-    std::map<std::string, std::string>* cf_stats) {
-  (*cf_stats)["io_stalls.level0_slowdown"] =
-      std::to_string(cf_stats_count_[L0_FILE_COUNT_LIMIT_SLOWDOWNS]);
-  (*cf_stats)["io_stalls.level0_slowdown_with_compaction"] =
-      std::to_string(cf_stats_count_[LOCKED_L0_FILE_COUNT_LIMIT_SLOWDOWNS]);
-  (*cf_stats)["io_stalls.level0_numfiles"] =
-      std::to_string(cf_stats_count_[L0_FILE_COUNT_LIMIT_STOPS]);
-  (*cf_stats)["io_stalls.level0_numfiles_with_compaction"] =
-      std::to_string(cf_stats_count_[LOCKED_L0_FILE_COUNT_LIMIT_STOPS]);
-  (*cf_stats)["io_stalls.stop_for_pending_compaction_bytes"] =
-      std::to_string(cf_stats_count_[PENDING_COMPACTION_BYTES_LIMIT_STOPS]);
-  (*cf_stats)["io_stalls.slowdown_for_pending_compaction_bytes"] =
-      std::to_string(cf_stats_count_[PENDING_COMPACTION_BYTES_LIMIT_SLOWDOWNS]);
-  (*cf_stats)["io_stalls.memtable_compaction"] =
-      std::to_string(cf_stats_count_[MEMTABLE_LIMIT_STOPS]);
-  (*cf_stats)["io_stalls.memtable_slowdown"] =
-      std::to_string(cf_stats_count_[MEMTABLE_LIMIT_SLOWDOWNS]);
+void InternalStats::DumpCFMapStatsWriteStall(
+    std::map<std::string, std::string>* value) {
+  uint64_t total_delays = 0;
+  uint64_t total_stops = 0;
+  constexpr uint32_t max_cf_scope_write_stall_cause =
+      static_cast<uint32_t>(WriteStallCause::kCFScopeWriteStallCauseEnumMax);
 
-  uint64_t total_stop = cf_stats_count_[L0_FILE_COUNT_LIMIT_STOPS] +
-                        cf_stats_count_[PENDING_COMPACTION_BYTES_LIMIT_STOPS] +
-                        cf_stats_count_[MEMTABLE_LIMIT_STOPS];
+  for (uint32_t i =
+           max_cf_scope_write_stall_cause - kNumCFScopeWriteStallCauses;
+       i < max_cf_scope_write_stall_cause; ++i) {
+    for (uint32_t j = 0;
+         j < static_cast<uint32_t>(WriteStallCondition::kNormal); ++j) {
+      WriteStallCause cause = static_cast<WriteStallCause>(i);
+      WriteStallCondition condition = static_cast<WriteStallCondition>(j);
+      InternalStats::InternalCFStatsType internal_cf_stat =
+          InternalCFStat(cause, condition);
 
-  uint64_t total_slowdown =
-      cf_stats_count_[L0_FILE_COUNT_LIMIT_SLOWDOWNS] +
-      cf_stats_count_[PENDING_COMPACTION_BYTES_LIMIT_SLOWDOWNS] +
-      cf_stats_count_[MEMTABLE_LIMIT_SLOWDOWNS];
+      if (internal_cf_stat == InternalStats::INTERNAL_CF_STATS_ENUM_MAX) {
+        continue;
+      }
 
-  (*cf_stats)["io_stalls.total_stop"] = std::to_string(total_stop);
-  (*cf_stats)["io_stalls.total_slowdown"] = std::to_string(total_slowdown);
+      std::string name =
+          WriteStallStatsMapKeys::CauseConditionCount(cause, condition);
+      uint64_t stat =
+          cf_stats_count_[static_cast<std::size_t>(internal_cf_stat)];
+      (*value)[name] = std::to_string(stat);
+
+      if (condition == WriteStallCondition::kDelayed) {
+        total_delays += stat;
+      } else if (condition == WriteStallCondition::kStopped) {
+        total_stops += stat;
+      }
+    }
+  }
+
+  (*value)[WriteStallStatsMapKeys::
+               CFL0FileCountLimitDelaysWithOngoingCompaction()] =
+      std::to_string(
+          cf_stats_count_[L0_FILE_COUNT_LIMIT_DELAYS_WITH_ONGOING_COMPACTION]);
+  (*value)[WriteStallStatsMapKeys::
+               CFL0FileCountLimitStopsWithOngoingCompaction()] =
+      std::to_string(
+          cf_stats_count_[L0_FILE_COUNT_LIMIT_STOPS_WITH_ONGOING_COMPACTION]);
+
+  (*value)[WriteStallStatsMapKeys::TotalStops()] = std::to_string(total_stops);
+  (*value)[WriteStallStatsMapKeys::TotalDelays()] =
+      std::to_string(total_delays);
+}
+
+void InternalStats::DumpCFStatsWriteStall(std::string* value,
+                                          uint64_t* total_stall_count) {
+  assert(value);
+
+  std::map<std::string, std::string> write_stall_stats_map;
+  DumpCFMapStatsWriteStall(&write_stall_stats_map);
+
+  std::ostringstream str;
+  str << "Write Stall (count): ";
+
+  for (auto write_stall_stats_map_iter = write_stall_stats_map.begin();
+       write_stall_stats_map_iter != write_stall_stats_map.end();
+       write_stall_stats_map_iter++) {
+    const auto& name_and_stat = *write_stall_stats_map_iter;
+    str << name_and_stat.first << ": " << name_and_stat.second;
+    if (std::next(write_stall_stats_map_iter) == write_stall_stats_map.end()) {
+      str << "\n";
+    } else {
+      str << ", ";
+    }
+  }
+
+  if (total_stall_count) {
+    *total_stall_count =
+        ParseUint64(
+            write_stall_stats_map[WriteStallStatsMapKeys::TotalStops()]) +
+        ParseUint64(
+            write_stall_stats_map[WriteStallStatsMapKeys::TotalDelays()]);
+    if (*total_stall_count > 0) {
+      str << "interval: " << *total_stall_count - cf_stats_snapshot_.stall_count
+          << " total count\n";
+    }
+  }
+  *value = str.str();
 }
 
 void InternalStats::DumpCFStats(std::string* value) {
-  DumpCFStatsNoFileHistogram(value);
+  DumpCFStatsNoFileHistogram(/*is_periodic=*/false, value);
   DumpCFFileHistogram(value);
 }
 
-void InternalStats::DumpCFStatsNoFileHistogram(std::string* value) {
+void InternalStats::DumpCFStatsNoFileHistogram(bool is_periodic,
+                                               std::string* value) {
   char buf[2000];
   // Per-ColumnFamily stats
   PrintLevelStatsHeader(buf, sizeof(buf), cfd_->GetName(), "Level");
@@ -1741,14 +1940,6 @@ void InternalStats::DumpCFStatsNoFileHistogram(std::string* value) {
   uint64_t ingest_l0_files_addfile =
       cf_stats_value_[INGESTED_LEVEL0_NUM_FILES_TOTAL];
   uint64_t ingest_keys_addfile = cf_stats_value_[INGESTED_NUM_KEYS_TOTAL];
-  // Cumulative summary
-  uint64_t total_stall_count =
-      cf_stats_count_[L0_FILE_COUNT_LIMIT_SLOWDOWNS] +
-      cf_stats_count_[L0_FILE_COUNT_LIMIT_STOPS] +
-      cf_stats_count_[PENDING_COMPACTION_BYTES_LIMIT_SLOWDOWNS] +
-      cf_stats_count_[PENDING_COMPACTION_BYTES_LIMIT_STOPS] +
-      cf_stats_count_[MEMTABLE_LIMIT_STOPS] +
-      cf_stats_count_[MEMTABLE_LIMIT_SLOWDOWNS];
   // Interval summary
   uint64_t interval_flush_ingest =
       flush_ingest - cf_stats_snapshot_.ingest_bytes_flush;
@@ -1864,47 +2055,27 @@ void InternalStats::DumpCFStatsNoFileHistogram(std::string* value) {
       interval_compact_bytes_read / kMB / std::max(interval_seconds_up, 0.001),
       interval_compact_micros / kMicrosInSec);
   value->append(buf);
-  cf_stats_snapshot_.compact_bytes_write = compact_bytes_write;
-  cf_stats_snapshot_.compact_bytes_read = compact_bytes_read;
-  cf_stats_snapshot_.compact_micros = compact_micros;
+  if (is_periodic) {
+    cf_stats_snapshot_.compact_bytes_write = compact_bytes_write;
+    cf_stats_snapshot_.compact_bytes_read = compact_bytes_read;
+    cf_stats_snapshot_.compact_micros = compact_micros;
+  }
 
-  snprintf(buf, sizeof(buf),
-           "Stalls(count): %" PRIu64
-           " level0_slowdown, "
-           "%" PRIu64
-           " level0_slowdown_with_compaction, "
-           "%" PRIu64
-           " level0_numfiles, "
-           "%" PRIu64
-           " level0_numfiles_with_compaction, "
-           "%" PRIu64
-           " stop for pending_compaction_bytes, "
-           "%" PRIu64
-           " slowdown for pending_compaction_bytes, "
-           "%" PRIu64
-           " memtable_compaction, "
-           "%" PRIu64
-           " memtable_slowdown, "
-           "interval %" PRIu64 " total count\n",
-           cf_stats_count_[L0_FILE_COUNT_LIMIT_SLOWDOWNS],
-           cf_stats_count_[LOCKED_L0_FILE_COUNT_LIMIT_SLOWDOWNS],
-           cf_stats_count_[L0_FILE_COUNT_LIMIT_STOPS],
-           cf_stats_count_[LOCKED_L0_FILE_COUNT_LIMIT_STOPS],
-           cf_stats_count_[PENDING_COMPACTION_BYTES_LIMIT_STOPS],
-           cf_stats_count_[PENDING_COMPACTION_BYTES_LIMIT_SLOWDOWNS],
-           cf_stats_count_[MEMTABLE_LIMIT_STOPS],
-           cf_stats_count_[MEMTABLE_LIMIT_SLOWDOWNS],
-           total_stall_count - cf_stats_snapshot_.stall_count);
-  value->append(buf);
+  std::string write_stall_stats;
+  uint64_t total_stall_count;
+  DumpCFStatsWriteStall(&write_stall_stats, &total_stall_count);
+  value->append(write_stall_stats);
 
-  cf_stats_snapshot_.seconds_up = seconds_up;
-  cf_stats_snapshot_.ingest_bytes_flush = flush_ingest;
-  cf_stats_snapshot_.ingest_bytes_addfile = add_file_ingest;
-  cf_stats_snapshot_.ingest_files_addfile = ingest_files_addfile;
-  cf_stats_snapshot_.ingest_l0_files_addfile = ingest_l0_files_addfile;
-  cf_stats_snapshot_.ingest_keys_addfile = ingest_keys_addfile;
-  cf_stats_snapshot_.comp_stats = compaction_stats_sum;
-  cf_stats_snapshot_.stall_count = total_stall_count;
+  if (is_periodic) {
+    cf_stats_snapshot_.seconds_up = seconds_up;
+    cf_stats_snapshot_.ingest_bytes_flush = flush_ingest;
+    cf_stats_snapshot_.ingest_bytes_addfile = add_file_ingest;
+    cf_stats_snapshot_.ingest_files_addfile = ingest_files_addfile;
+    cf_stats_snapshot_.ingest_l0_files_addfile = ingest_l0_files_addfile;
+    cf_stats_snapshot_.ingest_keys_addfile = ingest_keys_addfile;
+    cf_stats_snapshot_.comp_stats = compaction_stats_sum;
+    cf_stats_snapshot_.stall_count = total_stall_count;
+  }
 
   // Do not gather cache entry stats during CFStats because DB
   // mutex is held. Only dump last cached collection (rely on DB
@@ -1946,12 +2117,5 @@ void InternalStats::DumpCFFileHistogram(std::string* value) {
   value->append(oss.str());
 }
 
-#else
-
-const DBPropertyInfo* GetPropertyInfo(const Slice& /*property*/) {
-  return nullptr;
-}
-
-#endif  // !ROCKSDB_LITE
 
 }  // namespace ROCKSDB_NAMESPACE
