@@ -14,6 +14,7 @@
 #include <string>
 #include <vector>
 
+#include "cache/typed_cache.h"
 #include "db/dbformat.h"
 #include "db/range_del_aggregator.h"
 #include "options/cf_options.h"
@@ -56,6 +57,16 @@ class TableCache {
              const std::string& db_session_id);
   ~TableCache();
 
+  // Cache interface for table cache
+  using CacheInterface =
+      BasicTypedCacheInterface<TableReader, CacheEntryRole::kMisc>;
+  using TypedHandle = CacheInterface::TypedHandle;
+
+  // Cache interface for row cache
+  using RowCacheInterface =
+      BasicTypedCacheInterface<std::string, CacheEntryRole::kMisc>;
+  using RowHandle = RowCacheInterface::TypedHandle;
+
   // Return an iterator for the specified file number (the corresponding
   // file length must be exactly "file_size" bytes).  If "table_reader_ptr"
   // is non-nullptr, also sets "*table_reader_ptr" to point to the Table object
@@ -63,6 +74,11 @@ class TableCache {
   // the returned iterator.  The returned "*table_reader_ptr" object is owned
   // by the cache and should not be deleted, and is valid for as long as the
   // returned iterator is live.
+  // If !options.ignore_range_deletions, and range_del_iter is non-nullptr,
+  // then range_del_iter is set to a TruncatedRangeDelIterator for range
+  // tombstones in the SST file corresponding to the specified file number. The
+  // upper/lower bounds for the TruncatedRangeDelIterator are set to the SST
+  // file's boundary.
   // @param options Must outlive the returned iterator.
   // @param range_del_agg If non-nullptr, adds range deletions to the
   //    aggregator. If an error occurs, returns it in a NewErrorInternalIterator
@@ -79,7 +95,9 @@ class TableCache {
       TableReaderCaller caller, Arena* arena, bool skip_filters, int level,
       size_t max_file_size_for_l0_meta_pin,
       const InternalKey* smallest_compaction_key,
-      const InternalKey* largest_compaction_key, bool allow_unprepared_value);
+      const InternalKey* largest_compaction_key, bool allow_unprepared_value,
+      uint8_t protection_bytes_per_key,
+      TruncatedRangeDelIterator** range_del_iter = nullptr);
 
   // If a seek to internal key "k" in specified file finds an entry,
   // call get_context->SaveValue() repeatedly until
@@ -95,6 +113,7 @@ class TableCache {
       const ReadOptions& options,
       const InternalKeyComparator& internal_comparator,
       const FileMetaData& file_meta, const Slice& k, GetContext* get_context,
+      uint8_t block_protection_bytes_per_key,
       const std::shared_ptr<const SliceTransform>& prefix_extractor = nullptr,
       HistogramImpl* file_read_hist = nullptr, bool skip_filters = false,
       int level = -1, size_t max_file_size_for_l0_meta_pin = 0);
@@ -104,9 +123,23 @@ class TableCache {
   Status GetRangeTombstoneIterator(
       const ReadOptions& options,
       const InternalKeyComparator& internal_comparator,
-      const FileMetaData& file_meta,
+      const FileMetaData& file_meta, uint8_t block_protection_bytes_per_key,
       std::unique_ptr<FragmentedRangeTombstoneIterator>* out_iter);
 
+  // Call table reader's MultiGetFilter to use the bloom filter to filter out
+  // keys. Returns Status::NotSupported() if row cache needs to be checked.
+  // If the table cache is looked up to get the table reader, the cache handle
+  // is returned in table_handle. This handle should be passed back to
+  // MultiGet() so it can be released.
+  Status MultiGetFilter(
+      const ReadOptions& options,
+      const InternalKeyComparator& internal_comparator,
+      const FileMetaData& file_meta,
+      const std::shared_ptr<const SliceTransform>& prefix_extractor,
+      HistogramImpl* file_read_hist, int level,
+      MultiGetContext::Range* mget_range, TypedHandle** table_handle,
+      uint8_t block_protection_bytes_per_key);
+
   // If a seek to internal key "k" in specified file finds an entry,
   // call get_context->SaveValue() repeatedly until
   // it returns false. As a side effect, it will insert the TableReader
@@ -120,27 +153,23 @@ class TableCache {
       Status, MultiGet, const ReadOptions& options,
       const InternalKeyComparator& internal_comparator,
       const FileMetaData& file_meta, const MultiGetContext::Range* mget_range,
+      uint8_t block_protection_bytes_per_key,
       const std::shared_ptr<const SliceTransform>& prefix_extractor = nullptr,
       HistogramImpl* file_read_hist = nullptr, bool skip_filters = false,
-      int level = -1);
+      bool skip_range_deletions = false, int level = -1,
+      TypedHandle* table_handle = nullptr);
 
   // Evict any entry for the specified file number
   static void Evict(Cache* cache, uint64_t file_number);
 
-  // Query whether specified file number is currently in cache
-  static bool HasEntry(Cache* cache, uint64_t file_number);
-
-  // Clean table handle and erase it from the table cache
-  // Used in DB close, or the file is not live anymore.
-  void EraseHandle(const FileDescriptor& fd, Cache::Handle* handle);
-
   // Find table reader
   // @param skip_filters Disables loading/accessing the filter block
   // @param level == -1 means not specified
   Status FindTable(
       const ReadOptions& ro, const FileOptions& toptions,
       const InternalKeyComparator& internal_comparator,
-      const FileDescriptor& file_fd, Cache::Handle**,
+      const FileMetaData& file_meta, TypedHandle**,
+      uint8_t block_protection_bytes_per_key,
       const std::shared_ptr<const SliceTransform>& prefix_extractor = nullptr,
       const bool no_io = false, bool record_read_stats = true,
       HistogramImpl* file_read_hist = nullptr, bool skip_filters = false,
@@ -148,9 +177,6 @@ class TableCache {
       size_t max_file_size_for_l0_meta_pin = 0,
       Temperature file_temperature = Temperature::kUnknown);
 
-  // Get TableReader from a cache handle.
-  TableReader* GetTableReaderFromHandle(Cache::Handle* handle);
-
   // Get the table properties of a given table.
   // @no_io: indicates if we should load table to the cache if it is not present
   //         in table cache yet.
@@ -158,39 +184,46 @@ class TableCache {
   //            return Status::Incomplete() if table is not present in cache and
   //            we set `no_io` to be true.
   Status GetTableProperties(
-      const FileOptions& toptions,
+      const FileOptions& toptions, const ReadOptions& read_options,
       const InternalKeyComparator& internal_comparator,
-      const FileDescriptor& file_meta,
+      const FileMetaData& file_meta,
       std::shared_ptr<const TableProperties>* properties,
+      uint8_t block_protection_bytes_per_key,
       const std::shared_ptr<const SliceTransform>& prefix_extractor = nullptr,
       bool no_io = false);
 
+  Status ApproximateKeyAnchors(const ReadOptions& ro,
+                               const InternalKeyComparator& internal_comparator,
+                               const FileMetaData& file_meta,
+                               uint8_t block_protection_bytes_per_key,
+                               std::vector<TableReader::Anchor>& anchors);
+
   // Return total memory usage of the table reader of the file.
   // 0 if table reader of the file is not loaded.
   size_t GetMemoryUsageByTableReader(
-      const FileOptions& toptions,
+      const FileOptions& toptions, const ReadOptions& read_options,
       const InternalKeyComparator& internal_comparator,
-      const FileDescriptor& fd,
+      const FileMetaData& file_meta, uint8_t block_protection_bytes_per_key,
       const std::shared_ptr<const SliceTransform>& prefix_extractor = nullptr);
 
   // Returns approximated offset of a key in a file represented by fd.
   uint64_t ApproximateOffsetOf(
-      const Slice& key, const FileDescriptor& fd, TableReaderCaller caller,
+      const ReadOptions& read_options, const Slice& key,
+      const FileMetaData& file_meta, TableReaderCaller caller,
       const InternalKeyComparator& internal_comparator,
+      uint8_t block_protection_bytes_per_key,
       const std::shared_ptr<const SliceTransform>& prefix_extractor = nullptr);
 
   // Returns approximated data size between start and end keys in a file
   // represented by fd (the start key must not be greater than the end key).
   uint64_t ApproximateSize(
-      const Slice& start, const Slice& end, const FileDescriptor& fd,
-      TableReaderCaller caller,
+      const ReadOptions& read_options, const Slice& start, const Slice& end,
+      const FileMetaData& file_meta, TableReaderCaller caller,
       const InternalKeyComparator& internal_comparator,
+      uint8_t block_protection_bytes_per_key,
       const std::shared_ptr<const SliceTransform>& prefix_extractor = nullptr);
 
-  // Release the handle from a cache
-  void ReleaseHandle(Cache::Handle* handle);
-
-  Cache* get_cache() const { return cache_; }
+  CacheInterface& get_cache() { return cache_; }
 
   // Capacity of the backing Cache that indicates infinite TableCache capacity.
   // For example when max_open_files is -1 we set the backing Cache to this.
@@ -199,7 +232,7 @@ class TableCache {
   // The tables opened with this TableCache will be immortal, i.e., their
   // lifetime is as long as that of the DB.
   void SetTablesAreImmortal() {
-    if (cache_->GetCapacity() >= kInfiniteCapacity) {
+    if (cache_.get()->GetCapacity() >= kInfiniteCapacity) {
       immortal_tables_ = true;
     }
   }
@@ -209,7 +242,8 @@ class TableCache {
   Status GetTableReader(
       const ReadOptions& ro, const FileOptions& file_options,
       const InternalKeyComparator& internal_comparator,
-      const FileDescriptor& fd, bool sequential_mode, bool record_read_stats,
+      const FileMetaData& file_meta, bool sequential_mode,
+      bool record_read_stats, uint8_t block_protection_bytes_per_key,
       HistogramImpl* file_read_hist, std::unique_ptr<TableReader>* table_reader,
       const std::shared_ptr<const SliceTransform>& prefix_extractor = nullptr,
       bool skip_filters = false, int level = -1,
@@ -217,6 +251,11 @@ class TableCache {
       size_t max_file_size_for_l0_meta_pin = 0,
       Temperature file_temperature = Temperature::kUnknown);
 
+  // Update the max_covering_tombstone_seq in the GetContext for each key based
+  // on the range deletions in the table
+  void UpdateRangeTombstoneSeqnums(const ReadOptions& options, TableReader* t,
+                                   MultiGetContext::Range& table_range);
+
   // Create a key prefix for looking up the row cache. The prefix is of the
   // format row_cache_id + fd_number + seq_no. Later, the user key can be
   // appended to form the full key
@@ -232,7 +271,7 @@ class TableCache {
 
   const ImmutableOptions& ioptions_;
   const FileOptions& file_options_;
-  Cache* const cache_;
+  CacheInterface cache_;
   std::string row_cache_id_;
   bool immortal_tables_;
   BlockCacheTracer* const block_cache_tracer_;
