@@ -64,15 +64,16 @@ class VersionBuilderTest : public testing::Test {
            uint64_t num_entries = 0, uint64_t num_deletions = 0,
            bool sampled = false, SequenceNumber smallest_seqno = 0,
            SequenceNumber largest_seqno = 0,
-           uint64_t oldest_blob_file_number = kInvalidBlobFileNumber) {
+           uint64_t oldest_blob_file_number = kInvalidBlobFileNumber,
+           uint64_t epoch_number = kUnknownEpochNumber) {
     assert(level < vstorage_.num_levels());
     FileMetaData* f = new FileMetaData(
         file_number, path_id, file_size, GetInternalKey(smallest, smallest_seq),
         GetInternalKey(largest, largest_seq), smallest_seqno, largest_seqno,
         /* marked_for_compact */ false, Temperature::kUnknown,
         oldest_blob_file_number, kUnknownOldestAncesterTime,
-        kUnknownFileCreationTime, kUnknownFileChecksum,
-        kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+        kUnknownFileCreationTime, epoch_number, kUnknownFileChecksum,
+        kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
     f->compensated_file_size = file_size;
     f->num_entries = num_entries;
     f->num_deletions = num_deletions;
@@ -98,7 +99,8 @@ class VersionBuilderTest : public testing::Test {
     vstorage_.AddBlobFile(std::move(meta));
   }
 
-  void AddDummyFile(uint64_t table_file_number, uint64_t blob_file_number) {
+  void AddDummyFile(uint64_t table_file_number, uint64_t blob_file_number,
+                    uint64_t epoch_number) {
     constexpr int level = 0;
     constexpr char smallest[] = "bar";
     constexpr char largest[] = "foo";
@@ -112,11 +114,11 @@ class VersionBuilderTest : public testing::Test {
 
     Add(level, table_file_number, smallest, largest, file_size, path_id,
         smallest_seq, largest_seq, num_entries, num_deletions, sampled,
-        smallest_seq, largest_seq, blob_file_number);
+        smallest_seq, largest_seq, blob_file_number, epoch_number);
   }
 
   void AddDummyFileToEdit(VersionEdit* edit, uint64_t table_file_number,
-                          uint64_t blob_file_number) {
+                          uint64_t blob_file_number, uint64_t epoch_number) {
     assert(edit);
 
     constexpr int level = 0;
@@ -128,12 +130,13 @@ class VersionBuilderTest : public testing::Test {
     constexpr SequenceNumber largest_seqno = 300;
     constexpr bool marked_for_compaction = false;
 
-    edit->AddFile(
-        level, table_file_number, path_id, file_size, GetInternalKey(smallest),
-        GetInternalKey(largest), smallest_seqno, largest_seqno,
-        marked_for_compaction, Temperature::kUnknown, blob_file_number,
-        kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-        kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+    edit->AddFile(level, table_file_number, path_id, file_size,
+                  GetInternalKey(smallest), GetInternalKey(largest),
+                  smallest_seqno, largest_seqno, marked_for_compaction,
+                  Temperature::kUnknown, blob_file_number,
+                  kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                  epoch_number, kUnknownFileChecksum,
+                  kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
   }
 
   void UpdateVersionStorageInfo(VersionStorageInfo* vstorage) {
@@ -157,7 +160,13 @@ void UnrefFilesInVersion(VersionStorageInfo* new_vstorage) {
 }
 
 TEST_F(VersionBuilderTest, ApplyAndSaveTo) {
-  Add(0, 1U, "150", "200", 100U);
+  Add(0, 1U, "150", "200", 100U, /*path_id*/ 0,
+      /*smallest_seq*/ 100, /*largest_seq*/ 100,
+      /*num_entries*/ 0, /*num_deletions*/ 0,
+      /*sampled*/ false, /*smallest_seqno*/ 0,
+      /*largest_seqno*/ 0,
+      /*oldest_blob_file_number*/ kInvalidBlobFileNumber,
+      /*epoch_number*/ 1);
 
   Add(1, 66U, "150", "200", 100U);
   Add(1, 88U, "201", "300", 100U);
@@ -174,11 +183,12 @@ TEST_F(VersionBuilderTest, ApplyAndSaveTo) {
   UpdateVersionStorageInfo();
 
   VersionEdit version_edit;
-  version_edit.AddFile(
-      2, 666, 0, 100U, GetInternalKey("301"), GetInternalKey("350"), 200, 200,
-      false, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+  version_edit.AddFile(2, 666, 0, 100U, GetInternalKey("301"),
+                       GetInternalKey("350"), 200, 200, false,
+                       Temperature::kUnknown, kInvalidBlobFileNumber,
+                       kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                       kUnknownEpochNumber, kUnknownFileChecksum,
+                       kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
   version_edit.DeleteFile(3, 27U);
 
   EnvOptions env_options;
@@ -204,8 +214,12 @@ TEST_F(VersionBuilderTest, ApplyAndSaveTo) {
 TEST_F(VersionBuilderTest, ApplyAndSaveToDynamic) {
   ioptions_.level_compaction_dynamic_level_bytes = true;
 
-  Add(0, 1U, "150", "200", 100U, 0, 200U, 200U, 0, 0, false, 200U, 200U);
-  Add(0, 88U, "201", "300", 100U, 0, 100U, 100U, 0, 0, false, 100U, 100U);
+  Add(0, 1U, "150", "200", 100U, 0, 200U, 200U, 0, 0, false, 200U, 200U,
+      /*oldest_blob_file_number*/ kInvalidBlobFileNumber,
+      /*epoch_number*/ 2);
+  Add(0, 88U, "201", "300", 100U, 0, 100U, 100U, 0, 0, false, 100U, 100U,
+      /*oldest_blob_file_number*/ kInvalidBlobFileNumber,
+      /*epoch_number*/ 1);
 
   Add(4, 6U, "150", "179", 100U);
   Add(4, 7U, "180", "220", 100U);
@@ -217,11 +231,13 @@ TEST_F(VersionBuilderTest, ApplyAndSaveToDynamic) {
   UpdateVersionStorageInfo();
 
   VersionEdit version_edit;
-  version_edit.AddFile(
-      3, 666, 0, 100U, GetInternalKey("301"), GetInternalKey("350"), 200, 200,
-      false, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+  version_edit.AddFile(3, 666, 0, 100U, GetInternalKey("301"),
+                       GetInternalKey("350"), 200, 200, false,
+                       Temperature::kUnknown, kInvalidBlobFileNumber,
+                       kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                       kUnknownEpochNumber, kUnknownFileChecksum,
+                       kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
+
   version_edit.DeleteFile(0, 1U);
   version_edit.DeleteFile(0, 88U);
 
@@ -250,8 +266,12 @@ TEST_F(VersionBuilderTest, ApplyAndSaveToDynamic) {
 TEST_F(VersionBuilderTest, ApplyAndSaveToDynamic2) {
   ioptions_.level_compaction_dynamic_level_bytes = true;
 
-  Add(0, 1U, "150", "200", 100U, 0, 200U, 200U, 0, 0, false, 200U, 200U);
-  Add(0, 88U, "201", "300", 100U, 0, 100U, 100U, 0, 0, false, 100U, 100U);
+  Add(0, 1U, "150", "200", 100U, 0, 200U, 200U, 0, 0, false, 200U, 200U,
+      /*oldest_blob_file_number*/ kInvalidBlobFileNumber,
+      /*epoch_number*/ 2);
+  Add(0, 88U, "201", "300", 100U, 0, 100U, 100U, 0, 0, false, 100U, 100U,
+      /*oldest_blob_file_number*/ kInvalidBlobFileNumber,
+      /*epoch_number*/ 1);
 
   Add(4, 6U, "150", "179", 100U);
   Add(4, 7U, "180", "220", 100U);
@@ -263,11 +283,12 @@ TEST_F(VersionBuilderTest, ApplyAndSaveToDynamic2) {
   UpdateVersionStorageInfo();
 
   VersionEdit version_edit;
-  version_edit.AddFile(
-      4, 666, 0, 100U, GetInternalKey("301"), GetInternalKey("350"), 200, 200,
-      false, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+  version_edit.AddFile(4, 666, 0, 100U, GetInternalKey("301"),
+                       GetInternalKey("350"), 200, 200, false,
+                       Temperature::kUnknown, kInvalidBlobFileNumber,
+                       kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                       kUnknownEpochNumber, kUnknownFileChecksum,
+                       kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
   version_edit.DeleteFile(0, 1U);
   version_edit.DeleteFile(0, 88U);
   version_edit.DeleteFile(4, 6U);
@@ -299,31 +320,36 @@ TEST_F(VersionBuilderTest, ApplyMultipleAndSaveTo) {
   UpdateVersionStorageInfo();
 
   VersionEdit version_edit;
-  version_edit.AddFile(
-      2, 666, 0, 100U, GetInternalKey("301"), GetInternalKey("350"), 200, 200,
-      false, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
-  version_edit.AddFile(
-      2, 676, 0, 100U, GetInternalKey("401"), GetInternalKey("450"), 200, 200,
-      false, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
-  version_edit.AddFile(
-      2, 636, 0, 100U, GetInternalKey("601"), GetInternalKey("650"), 200, 200,
-      false, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
-  version_edit.AddFile(
-      2, 616, 0, 100U, GetInternalKey("501"), GetInternalKey("550"), 200, 200,
-      false, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
-  version_edit.AddFile(
-      2, 606, 0, 100U, GetInternalKey("701"), GetInternalKey("750"), 200, 200,
-      false, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+  version_edit.AddFile(2, 666, 0, 100U, GetInternalKey("301"),
+                       GetInternalKey("350"), 200, 200, false,
+                       Temperature::kUnknown, kInvalidBlobFileNumber,
+                       kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                       kUnknownEpochNumber, kUnknownFileChecksum,
+                       kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
+  version_edit.AddFile(2, 676, 0, 100U, GetInternalKey("401"),
+                       GetInternalKey("450"), 200, 200, false,
+                       Temperature::kUnknown, kInvalidBlobFileNumber,
+                       kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                       kUnknownEpochNumber, kUnknownFileChecksum,
+                       kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
+  version_edit.AddFile(2, 636, 0, 100U, GetInternalKey("601"),
+                       GetInternalKey("650"), 200, 200, false,
+                       Temperature::kUnknown, kInvalidBlobFileNumber,
+                       kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                       kUnknownEpochNumber, kUnknownFileChecksum,
+                       kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
+  version_edit.AddFile(2, 616, 0, 100U, GetInternalKey("501"),
+                       GetInternalKey("550"), 200, 200, false,
+                       Temperature::kUnknown, kInvalidBlobFileNumber,
+                       kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                       kUnknownEpochNumber, kUnknownFileChecksum,
+                       kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
+  version_edit.AddFile(2, 606, 0, 100U, GetInternalKey("701"),
+                       GetInternalKey("750"), 200, 200, false,
+                       Temperature::kUnknown, kInvalidBlobFileNumber,
+                       kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                       kUnknownEpochNumber, kUnknownFileChecksum,
+                       kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
 
   EnvOptions env_options;
   constexpr TableCache* table_cache = nullptr;
@@ -358,46 +384,53 @@ TEST_F(VersionBuilderTest, ApplyDeleteAndSaveTo) {
                                   kCompactionStyleLevel, nullptr, false);
 
   VersionEdit version_edit;
-  version_edit.AddFile(
-      2, 666, 0, 100U, GetInternalKey("301"), GetInternalKey("350"), 200, 200,
-      false, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
-  version_edit.AddFile(
-      2, 676, 0, 100U, GetInternalKey("401"), GetInternalKey("450"), 200, 200,
-      false, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
-  version_edit.AddFile(
-      2, 636, 0, 100U, GetInternalKey("601"), GetInternalKey("650"), 200, 200,
-      false, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
-  version_edit.AddFile(
-      2, 616, 0, 100U, GetInternalKey("501"), GetInternalKey("550"), 200, 200,
-      false, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
-  version_edit.AddFile(
-      2, 606, 0, 100U, GetInternalKey("701"), GetInternalKey("750"), 200, 200,
-      false, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+  version_edit.AddFile(2, 666, 0, 100U, GetInternalKey("301"),
+                       GetInternalKey("350"), 200, 200, false,
+                       Temperature::kUnknown, kInvalidBlobFileNumber,
+                       kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                       kUnknownEpochNumber, kUnknownFileChecksum,
+                       kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
+  version_edit.AddFile(2, 676, 0, 100U, GetInternalKey("401"),
+                       GetInternalKey("450"), 200, 200, false,
+                       Temperature::kUnknown, kInvalidBlobFileNumber,
+                       kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                       kUnknownEpochNumber, kUnknownFileChecksum,
+                       kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
+  version_edit.AddFile(2, 636, 0, 100U, GetInternalKey("601"),
+                       GetInternalKey("650"), 200, 200, false,
+                       Temperature::kUnknown, kInvalidBlobFileNumber,
+                       kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                       kUnknownEpochNumber, kUnknownFileChecksum,
+                       kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
+  version_edit.AddFile(2, 616, 0, 100U, GetInternalKey("501"),
+                       GetInternalKey("550"), 200, 200, false,
+                       Temperature::kUnknown, kInvalidBlobFileNumber,
+                       kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                       kUnknownEpochNumber, kUnknownFileChecksum,
+                       kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
+  version_edit.AddFile(2, 606, 0, 100U, GetInternalKey("701"),
+                       GetInternalKey("750"), 200, 200, false,
+                       Temperature::kUnknown, kInvalidBlobFileNumber,
+                       kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                       kUnknownEpochNumber, kUnknownFileChecksum,
+                       kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
   ASSERT_OK(version_builder.Apply(&version_edit));
 
   VersionEdit version_edit2;
-  version_edit.AddFile(
-      2, 808, 0, 100U, GetInternalKey("901"), GetInternalKey("950"), 200, 200,
-      false, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+  version_edit.AddFile(2, 808, 0, 100U, GetInternalKey("901"),
+                       GetInternalKey("950"), 200, 200, false,
+                       Temperature::kUnknown, kInvalidBlobFileNumber,
+                       kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                       kUnknownEpochNumber, kUnknownFileChecksum,
+                       kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
   version_edit2.DeleteFile(2, 616);
   version_edit2.DeleteFile(2, 636);
-  version_edit.AddFile(
-      2, 806, 0, 100U, GetInternalKey("801"), GetInternalKey("850"), 200, 200,
-      false, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+  version_edit.AddFile(2, 806, 0, 100U, GetInternalKey("801"),
+                       GetInternalKey("850"), 200, 200, false,
+                       Temperature::kUnknown, kInvalidBlobFileNumber,
+                       kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                       kUnknownEpochNumber, kUnknownFileChecksum,
+                       kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
 
   ASSERT_OK(version_builder.Apply(&version_edit2));
   ASSERT_OK(version_builder.SaveTo(&new_vstorage));
@@ -507,8 +540,9 @@ TEST_F(VersionBuilderTest, ApplyFileDeletionAndAddition) {
                    GetInternalKey(largest, largest_seq), smallest_seqno,
                    largest_seqno, marked_for_compaction, Temperature::kUnknown,
                    kInvalidBlobFileNumber, kUnknownOldestAncesterTime,
-                   kUnknownFileCreationTime, kUnknownFileChecksum,
-                   kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+                   kUnknownFileCreationTime, kUnknownEpochNumber,
+                   kUnknownFileChecksum, kUnknownFileChecksumFuncName,
+                   kNullUniqueId64x2, 0, 0);
 
   ASSERT_OK(builder.Apply(&addition));
 
@@ -552,12 +586,13 @@ TEST_F(VersionBuilderTest, ApplyFileAdditionAlreadyInBase) {
   constexpr SequenceNumber largest_seqno = 1000;
   constexpr bool marked_for_compaction = false;
 
-  edit.AddFile(
-      new_level, file_number, path_id, file_size, GetInternalKey(smallest),
-      GetInternalKey(largest), smallest_seqno, largest_seqno,
-      marked_for_compaction, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+  edit.AddFile(new_level, file_number, path_id, file_size,
+               GetInternalKey(smallest), GetInternalKey(largest),
+               smallest_seqno, largest_seqno, marked_for_compaction,
+               Temperature::kUnknown, kInvalidBlobFileNumber,
+               kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+               kUnknownEpochNumber, kUnknownFileChecksum,
+               kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
 
   const Status s = builder.Apply(&edit);
   ASSERT_TRUE(s.IsCorruption());
@@ -592,8 +627,9 @@ TEST_F(VersionBuilderTest, ApplyFileAdditionAlreadyApplied) {
                GetInternalKey(largest), smallest_seqno, largest_seqno,
                marked_for_compaction, Temperature::kUnknown,
                kInvalidBlobFileNumber, kUnknownOldestAncesterTime,
-               kUnknownFileCreationTime, kUnknownFileChecksum,
-               kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+               kUnknownFileCreationTime, kUnknownEpochNumber,
+               kUnknownFileChecksum, kUnknownFileChecksumFuncName,
+               kNullUniqueId64x2, 0, 0);
 
   ASSERT_OK(builder.Apply(&edit));
 
@@ -601,12 +637,13 @@ TEST_F(VersionBuilderTest, ApplyFileAdditionAlreadyApplied) {
 
   constexpr int new_level = 2;
 
-  other_edit.AddFile(
-      new_level, file_number, path_id, file_size, GetInternalKey(smallest),
-      GetInternalKey(largest), smallest_seqno, largest_seqno,
-      marked_for_compaction, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+  other_edit.AddFile(new_level, file_number, path_id, file_size,
+                     GetInternalKey(smallest), GetInternalKey(largest),
+                     smallest_seqno, largest_seqno, marked_for_compaction,
+                     Temperature::kUnknown, kInvalidBlobFileNumber,
+                     kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                     kUnknownEpochNumber, kUnknownFileChecksum,
+                     kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
 
   const Status s = builder.Apply(&other_edit);
   ASSERT_TRUE(s.IsCorruption());
@@ -637,12 +674,13 @@ TEST_F(VersionBuilderTest, ApplyFileAdditionAndDeletion) {
 
   VersionEdit addition;
 
-  addition.AddFile(
-      level, file_number, path_id, file_size, GetInternalKey(smallest),
-      GetInternalKey(largest), smallest_seqno, largest_seqno,
-      marked_for_compaction, Temperature::kUnknown, kInvalidBlobFileNumber,
-      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-      kUnknownFileChecksum, kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+  addition.AddFile(level, file_number, path_id, file_size,
+                   GetInternalKey(smallest), GetInternalKey(largest),
+                   smallest_seqno, largest_seqno, marked_for_compaction,
+                   Temperature::kUnknown, kInvalidBlobFileNumber,
+                   kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+                   kUnknownEpochNumber, kUnknownFileChecksum,
+                   kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
 
   ASSERT_OK(builder.Apply(&addition));
 
@@ -691,7 +729,8 @@ TEST_F(VersionBuilderTest, ApplyBlobFileAddition) {
 
   // Add dummy table file to ensure the blob file is referenced.
   constexpr uint64_t table_file_number = 1;
-  AddDummyFileToEdit(&edit, table_file_number, blob_file_number);
+  AddDummyFileToEdit(&edit, table_file_number, blob_file_number,
+                     1 /*epoch_number*/);
 
   ASSERT_OK(builder.Apply(&edit));
 
@@ -813,7 +852,7 @@ TEST_F(VersionBuilderTest, ApplyBlobFileGarbageFileInBase) {
   ASSERT_NE(meta, nullptr);
 
   // Add dummy table file to ensure the blob file is referenced.
-  AddDummyFile(table_file_number, blob_file_number);
+  AddDummyFile(table_file_number, blob_file_number, 1 /*epoch_number*/);
 
   UpdateVersionStorageInfo();
 
@@ -892,7 +931,8 @@ TEST_F(VersionBuilderTest, ApplyBlobFileGarbageFileAdditionApplied) {
 
   // Add dummy table file to ensure the blob file is referenced.
   constexpr uint64_t table_file_number = 1;
-  AddDummyFileToEdit(&addition, table_file_number, blob_file_number);
+  AddDummyFileToEdit(&addition, table_file_number, blob_file_number,
+                     1 /*epoch_number*/);
 
   ASSERT_OK(builder.Apply(&addition));
 
@@ -989,7 +1029,8 @@ TEST_F(VersionBuilderTest, BlobFileGarbageOverflow) {
 
   // Add dummy table file to ensure the blob file is referenced.
   constexpr uint64_t table_file_number = 1;
-  AddDummyFileToEdit(&addition, table_file_number, blob_file_number);
+  AddDummyFileToEdit(&addition, table_file_number, blob_file_number,
+                     1 /*epoch_number*/);
 
   ASSERT_OK(builder.Apply(&addition));
 
@@ -1050,7 +1091,7 @@ TEST_F(VersionBuilderTest, SaveBlobFilesTo) {
     const uint64_t table_file_number = 2 * i;
     const uint64_t blob_file_number = 2 * i + 1;
 
-    AddDummyFile(table_file_number, blob_file_number);
+    AddDummyFile(table_file_number, blob_file_number, i /*epoch_number*/);
   }
 
   UpdateVersionStorageInfo();
@@ -1171,7 +1212,8 @@ TEST_F(VersionBuilderTest, SaveBlobFilesToConcurrentJobs) {
   constexpr uint64_t garbage_blob_count = 0;
   constexpr uint64_t garbage_blob_bytes = 0;
 
-  AddDummyFile(base_table_file_number, base_blob_file_number);
+  AddDummyFile(base_table_file_number, base_blob_file_number,
+               1 /*epoch_number*/);
   AddBlob(base_blob_file_number, base_total_blob_count, base_total_blob_bytes,
           checksum_method, checksum_value,
           BlobFileMetaData::LinkedSsts{base_table_file_number},
@@ -1206,12 +1248,12 @@ TEST_F(VersionBuilderTest, SaveBlobFilesToConcurrentJobs) {
   constexpr uint64_t total_blob_count = 234;
   constexpr uint64_t total_blob_bytes = 1 << 22;
 
-  edit.AddFile(level, table_file_number, path_id, file_size,
-               GetInternalKey(smallest), GetInternalKey(largest),
-               smallest_seqno, largest_seqno, marked_for_compaction,
-               Temperature::kUnknown, blob_file_number,
-               kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-               checksum_value, checksum_method, kNullUniqueId64x2);
+  edit.AddFile(
+      level, table_file_number, path_id, file_size, GetInternalKey(smallest),
+      GetInternalKey(largest), smallest_seqno, largest_seqno,
+      marked_for_compaction, Temperature::kUnknown, blob_file_number,
+      kUnknownOldestAncesterTime, kUnknownFileCreationTime, 2 /*epoch_number*/,
+      checksum_value, checksum_method, kNullUniqueId64x2, 0, 0);
   edit.AddBlobFile(blob_file_number, total_blob_count, total_blob_bytes,
                    checksum_method, checksum_value);
 
@@ -1297,8 +1339,9 @@ TEST_F(VersionBuilderTest, CheckConsistencyForBlobFiles) {
                /* largest_seqno */ 200, /* marked_for_compaction */ false,
                Temperature::kUnknown,
                /* oldest_blob_file_number */ 16, kUnknownOldestAncesterTime,
-               kUnknownFileCreationTime, kUnknownFileChecksum,
-               kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+               kUnknownFileCreationTime, kUnknownEpochNumber,
+               kUnknownFileChecksum, kUnknownFileChecksumFuncName,
+               kNullUniqueId64x2, 0, 0);
 
   edit.AddFile(/* level */ 1, /* file_number */ 700, /* path_id */ 0,
                /* file_size */ 100, /* smallest */ GetInternalKey("801"),
@@ -1306,8 +1349,9 @@ TEST_F(VersionBuilderTest, CheckConsistencyForBlobFiles) {
                /* largest_seqno */ 200, /* marked_for_compaction */ false,
                Temperature::kUnknown,
                /* oldest_blob_file_number */ 1000, kUnknownOldestAncesterTime,
-               kUnknownFileCreationTime, kUnknownFileChecksum,
-               kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+               kUnknownFileCreationTime, kUnknownEpochNumber,
+               kUnknownFileChecksum, kUnknownFileChecksumFuncName,
+               kNullUniqueId64x2, 0, 0);
   edit.AddBlobFile(/* blob_file_number */ 1000, /* total_blob_count */ 2000,
                    /* total_blob_bytes */ 200000,
                    /* checksum_method */ std::string(),
@@ -1527,8 +1571,8 @@ TEST_F(VersionBuilderTest, MaintainLinkedSstsForBlobFiles) {
       /* largest_seqno */ 2100, /* marked_for_compaction */ false,
       Temperature::kUnknown,
       /* oldest_blob_file_number */ 1, kUnknownOldestAncesterTime,
-      kUnknownFileCreationTime, kUnknownFileChecksum,
-      kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+      kUnknownFileCreationTime, kUnknownEpochNumber, kUnknownFileChecksum,
+      kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
 
   // Add an SST that does not reference any blob files.
   edit.AddFile(
@@ -1537,8 +1581,8 @@ TEST_F(VersionBuilderTest, MaintainLinkedSstsForBlobFiles) {
       /* largest */ GetInternalKey("22", 2200), /* smallest_seqno */ 2200,
       /* largest_seqno */ 2200, /* marked_for_compaction */ false,
       Temperature::kUnknown, kInvalidBlobFileNumber, kUnknownOldestAncesterTime,
-      kUnknownFileCreationTime, kUnknownFileChecksum,
-      kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+      kUnknownFileCreationTime, kUnknownEpochNumber, kUnknownFileChecksum,
+      kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
 
   // Delete a file that references a blob file.
   edit.DeleteFile(/* level */ 1, /* file_number */ 6);
@@ -1559,8 +1603,9 @@ TEST_F(VersionBuilderTest, MaintainLinkedSstsForBlobFiles) {
                /* largest_seqno */ 300, /* marked_for_compaction */ false,
                Temperature::kUnknown,
                /* oldest_blob_file_number */ 3, kUnknownOldestAncesterTime,
-               kUnknownFileCreationTime, kUnknownFileChecksum,
-               kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+               kUnknownFileCreationTime, kUnknownEpochNumber,
+               kUnknownFileChecksum, kUnknownFileChecksumFuncName,
+               kNullUniqueId64x2, 0, 0);
 
   // Trivially move a file that does not reference any blob files.
   edit.DeleteFile(/* level */ 1, /* file_number */ 13);
@@ -1571,8 +1616,8 @@ TEST_F(VersionBuilderTest, MaintainLinkedSstsForBlobFiles) {
                /* largest_seqno */ 1300, /* marked_for_compaction */ false,
                Temperature::kUnknown, kInvalidBlobFileNumber,
                kUnknownOldestAncesterTime, kUnknownFileCreationTime,
-               kUnknownFileChecksum, kUnknownFileChecksumFuncName,
-               kNullUniqueId64x2);
+               kUnknownEpochNumber, kUnknownFileChecksum,
+               kUnknownFileChecksumFuncName, kNullUniqueId64x2, 0, 0);
 
   // Add one more SST file that references a blob file, then promptly
   // delete it in a second version edit before the new version gets saved.
@@ -1584,8 +1629,9 @@ TEST_F(VersionBuilderTest, MaintainLinkedSstsForBlobFiles) {
                /* largest_seqno */ 2300, /* marked_for_compaction */ false,
                Temperature::kUnknown,
                /* oldest_blob_file_number */ 5, kUnknownOldestAncesterTime,
-               kUnknownFileCreationTime, kUnknownFileChecksum,
-               kUnknownFileChecksumFuncName, kNullUniqueId64x2);
+               kUnknownFileCreationTime, kUnknownEpochNumber,
+               kUnknownFileChecksum, kUnknownFileChecksumFuncName,
+               kNullUniqueId64x2, 0, 0);
 
   VersionEdit edit2;
 
@@ -1634,7 +1680,13 @@ TEST_F(VersionBuilderTest, MaintainLinkedSstsForBlobFiles) {
 }
 
 TEST_F(VersionBuilderTest, CheckConsistencyForFileDeletedTwice) {
-  Add(0, 1U, "150", "200", 100U);
+  Add(0, 1U, "150", "200", 100, /*path_id*/ 0,
+      /*smallest_seq*/ 100, /*largest_seq*/ 100,
+      /*num_entries*/ 0, /*num_deletions*/ 0,
+      /*sampled*/ false, /*smallest_seqno*/ 0,
+      /*largest_seqno*/ 0,
+      /*oldest_blob_file_number*/ kInvalidBlobFileNumber,
+      /*epoch_number*/ 1);
 
   UpdateVersionStorageInfo();
 
@@ -1656,16 +1708,109 @@ TEST_F(VersionBuilderTest, CheckConsistencyForFileDeletedTwice) {
   UpdateVersionStorageInfo(&new_vstorage);
 
   VersionBuilder version_builder2(env_options, &ioptions_, table_cache,
-                                 &new_vstorage, version_set);
+                                  &new_vstorage, version_set);
   VersionStorageInfo new_vstorage2(&icmp_, ucmp_, options_.num_levels,
-                                  kCompactionStyleLevel, nullptr,
-                                  true /* force_consistency_checks */);
+                                   kCompactionStyleLevel, nullptr,
+                                   true /* force_consistency_checks */);
   ASSERT_NOK(version_builder2.Apply(&version_edit));
 
   UnrefFilesInVersion(&new_vstorage);
   UnrefFilesInVersion(&new_vstorage2);
 }
 
+TEST_F(VersionBuilderTest, CheckConsistencyForL0FilesSortedByEpochNumber) {
+  Status s;
+  // To verify files of same epoch number of overlapping ranges are caught as
+  // corrupted
+  VersionEdit version_edit_1;
+  version_edit_1.AddFile(
+      /* level */ 0, /* file_number */ 1U, /* path_id */ 0,
+      /* file_size */ 100, /* smallest */ GetInternalKey("a", 1),
+      /* largest */ GetInternalKey("c", 3), /* smallest_seqno */ 1,
+      /* largest_seqno */ 3, /* marked_for_compaction */ false,
+      Temperature::kUnknown,
+      /* oldest_blob_file_number */ kInvalidBlobFileNumber,
+      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+      1 /* epoch_number */, kUnknownFileChecksum, kUnknownFileChecksumFuncName,
+      kNullUniqueId64x2, 0, 0);
+  version_edit_1.AddFile(
+      /* level */ 0, /* file_number */ 2U, /* path_id */ 0,
+      /* file_size */ 100, /* smallest */ GetInternalKey("b", 2),
+      /* largest */ GetInternalKey("d", 4), /* smallest_seqno */ 2,
+      /* largest_seqno */ 4, /* marked_for_compaction */ false,
+      Temperature::kUnknown,
+      /* oldest_blob_file_number */ kInvalidBlobFileNumber,
+      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+      1 /* epoch_number */, kUnknownFileChecksum, kUnknownFileChecksumFuncName,
+      kNullUniqueId64x2, 0, 0);
+
+  VersionBuilder version_builder_1(EnvOptions(), &ioptions_,
+                                   nullptr /* table_cache */, &vstorage_,
+                                   nullptr /* file_metadata_cache_res_mgr */);
+  VersionStorageInfo new_vstorage_1(
+      &icmp_, ucmp_, options_.num_levels, kCompactionStyleLevel,
+      nullptr /* src_vstorage */, true /* force_consistency_checks */);
+
+  ASSERT_OK(version_builder_1.Apply(&version_edit_1));
+  s = version_builder_1.SaveTo(&new_vstorage_1);
+  EXPECT_TRUE(s.IsCorruption());
+  EXPECT_TRUE(std::strstr(
+      s.getState(), "L0 files of same epoch number but overlapping range"));
+  UnrefFilesInVersion(&new_vstorage_1);
+
+  // To verify L0 files not sorted by epoch_number are caught as corrupted
+  VersionEdit version_edit_2;
+  version_edit_2.AddFile(
+      /* level */ 0, /* file_number */ 1U, /* path_id */ 0,
+      /* file_size */ 100, /* smallest */ GetInternalKey("a", 1),
+      /* largest */ GetInternalKey("a", 1), /* smallest_seqno */ 1,
+      /* largest_seqno */ 1, /* marked_for_compaction */ false,
+      Temperature::kUnknown,
+      /* oldest_blob_file_number */ kInvalidBlobFileNumber,
+      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+      1 /* epoch_number */, kUnknownFileChecksum, kUnknownFileChecksumFuncName,
+      kNullUniqueId64x2, 0, 0);
+  version_edit_2.AddFile(
+      /* level */ 0, /* file_number */ 2U, /* path_id */ 0,
+      /* file_size */ 100, /* smallest */ GetInternalKey("b", 2),
+      /* largest */ GetInternalKey("b", 2), /* smallest_seqno */ 2,
+      /* largest_seqno */ 2, /* marked_for_compaction */ false,
+      Temperature::kUnknown,
+      /* oldest_blob_file_number */ kInvalidBlobFileNumber,
+      kUnknownOldestAncesterTime, kUnknownFileCreationTime,
+      2 /* epoch_number */, kUnknownFileChecksum, kUnknownFileChecksumFuncName,
+      kNullUniqueId64x2, 0, 0);
+
+  VersionBuilder version_builder_2(EnvOptions(), &ioptions_,
+                                   nullptr /* table_cache */, &vstorage_,
+                                   nullptr /* file_metadata_cache_res_mgr */);
+  VersionStorageInfo new_vstorage_2(
+      &icmp_, ucmp_, options_.num_levels, kCompactionStyleLevel,
+      nullptr /* src_vstorage */, true /* force_consistency_checks */);
+
+  ASSERT_OK(version_builder_2.Apply(&version_edit_2));
+  s = version_builder_2.SaveTo(&new_vstorage_2);
+  ASSERT_TRUE(s.ok());
+
+  const std::vector<FileMetaData*>& l0_files = new_vstorage_2.LevelFiles(0);
+  ASSERT_EQ(l0_files.size(), 2);
+  // Manually corrupt L0 files's epoch_number
+  l0_files[0]->epoch_number = 1;
+  l0_files[1]->epoch_number = 2;
+
+  // To surface corruption error by applying dummy version edit
+  VersionEdit dummy_version_edit;
+  VersionBuilder dummy_version_builder(
+      EnvOptions(), &ioptions_, nullptr /* table_cache */, &vstorage_,
+      nullptr /* file_metadata_cache_res_mgr */);
+  ASSERT_OK(dummy_version_builder.Apply(&dummy_version_edit));
+  s = dummy_version_builder.SaveTo(&new_vstorage_2);
+  EXPECT_TRUE(s.IsCorruption());
+  EXPECT_TRUE(std::strstr(s.getState(), "L0 files are not sorted properly"));
+
+  UnrefFilesInVersion(&new_vstorage_2);
+}
+
 TEST_F(VersionBuilderTest, EstimatedActiveKeys) {
   const uint32_t kTotalSamples = 20;
   const uint32_t kNumLevels = 5;
