@@ -18,7 +18,6 @@
 #include "test_util/testharness.h"
 #include "util/string_util.h"
 
-#ifndef ROCKSDB_LITE
 
 namespace ROCKSDB_NAMESPACE {
 
@@ -136,7 +135,7 @@ TEST_F(DeleteSchedulerTest, BasicRateLimiting) {
         EXPECT_EQ(dummy_files_dirs_[0], *dir);
       });
 
-  int num_files = 100;  // 100 files
+  int num_files = 100;        // 100 files
   uint64_t file_size = 1024;  // every file is 1 kb
   std::vector<uint64_t> delete_kbs_per_sec = {512, 200, 100, 50, 25};
 
@@ -249,7 +248,7 @@ TEST_F(DeleteSchedulerTest, RateLimitingMultiThreaded) {
       [&](void* arg) { penalties.push_back(*(static_cast<uint64_t*>(arg))); });
 
   int thread_cnt = 10;
-  int num_files = 10;  // 10 files per thread
+  int num_files = 10;         // 10 files per thread
   uint64_t file_size = 1024;  // every file is 1 kb
 
   std::vector<uint64_t> delete_kbs_per_sec = {512, 200, 100, 50, 25};
@@ -591,8 +590,7 @@ TEST_F(DeleteSchedulerTest, DISABLED_DynamicRateLimiting1) {
   rate_bytes_per_sec_ = 0;  // Disable rate limiting initially
   NewDeleteScheduler();
 
-
-  int num_files = 10;  // 10 files
+  int num_files = 10;         // 10 files
   uint64_t file_size = 1024;  // every file is 1 kb
 
   std::vector<int64_t> delete_kbs_per_sec = {512, 200, 0, 100, 50, -2, 25};
@@ -662,9 +660,9 @@ TEST_F(DeleteSchedulerTest, ImmediateDeleteOn25PercDBSize) {
 
   ROCKSDB_NAMESPACE::SyncPoint::GetInstance()->EnableProcessing();
 
-  int num_files = 100;  // 100 files
-  uint64_t file_size = 1024 * 10; // 100 KB as a file size
-  rate_bytes_per_sec_ = 1;  // 1 byte per sec (very slow trash delete)
+  int num_files = 100;             // 100 files
+  uint64_t file_size = 1024 * 10;  // 100 KB as a file size
+  rate_bytes_per_sec_ = 1;         // 1 byte per sec (very slow trash delete)
 
   NewDeleteScheduler();
   delete_scheduler_->SetMaxTrashDBRatio(0.25);
@@ -717,9 +715,3 @@ int main(int argc, char** argv) {
   return RUN_ALL_TESTS();
 }
 
-#else
-int main(int /*argc*/, char** /*argv*/) {
-  printf("DeleteScheduler is not supported in ROCKSDB_LITE\n");
-  return 0;
-}
-#endif  // ROCKSDB_LITE
