@@ -116,6 +116,8 @@ struct IOOptions {
   // directories and list only files in GetChildren API.
   bool do_not_recurse;
 
+  Env::IOActivity io_activity = Env::IOActivity::kUnknown;
+
   IOOptions() : IOOptions(false) {}
 
   explicit IOOptions(bool force_dir_fsync_)
@@ -272,12 +274,6 @@ class FileSystem : public Customizable {
   static const char* Type() { return "FileSystem"; }
   static const char* kDefaultName() { return "DefaultFileSystem"; }
 
-  // Loads the FileSystem specified by the input value into the result
-  // The CreateFromString alternative should be used; this method may be
-  // deprecated in a future release.
-  static Status Load(const std::string& value,
-                     std::shared_ptr<FileSystem>* result);
-
   // Loads the FileSystem specified by the input value into the result
   // @see Customizable for a more detailed description of the parameters and
   // return codes
@@ -344,8 +340,7 @@ class FileSystem : public Customizable {
   // The returned file may be concurrently accessed by multiple threads.
   virtual IOStatus NewRandomAccessFile(
       const std::string& fname, const FileOptions& file_opts,
-      std::unique_ptr<FSRandomAccessFile>* result,
-      IODebugContext* dbg) = 0;
+      std::unique_ptr<FSRandomAccessFile>* result, IODebugContext* dbg) = 0;
   // These values match Linux definition
   // https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git/tree/include/uapi/linux/fcntl.h#n56
   enum WriteLifeTimeHint {
@@ -497,7 +492,8 @@ class FileSystem : public Customizable {
   virtual IOStatus Truncate(const std::string& /*fname*/, size_t /*size*/,
                             const IOOptions& /*options*/,
                             IODebugContext* /*dbg*/) {
-    return IOStatus::NotSupported("Truncate is not supported for this FileSystem");
+    return IOStatus::NotSupported(
+        "Truncate is not supported for this FileSystem");
   }
 
   // Create the specified directory. Returns error if directory exists.
@@ -534,7 +530,8 @@ class FileSystem : public Customizable {
                             const std::string& /*target*/,
                             const IOOptions& /*options*/,
                             IODebugContext* /*dbg*/) {
-    return IOStatus::NotSupported("LinkFile is not supported for this FileSystem");
+    return IOStatus::NotSupported(
+        "LinkFile is not supported for this FileSystem");
   }
 
   virtual IOStatus NumFileLinks(const std::string& /*fname*/,
@@ -548,7 +545,8 @@ class FileSystem : public Customizable {
                                 const std::string& /*second*/,
                                 const IOOptions& /*options*/, bool* /*res*/,
                                 IODebugContext* /*dbg*/) {
-    return IOStatus::NotSupported("AreFilesSame is not supported for this FileSystem");
+    return IOStatus::NotSupported(
+        "AreFilesSame is not supported for this FileSystem");
   }
 
   // Lock the specified file.  Used to prevent concurrent access to
@@ -612,7 +610,7 @@ class FileSystem : public Customizable {
   // the FileOptions in the parameters, but is optimized for writing log files.
   // Default implementation returns the copy of the same object.
   virtual FileOptions OptimizeForLogWrite(const FileOptions& file_options,
-                                         const DBOptions& db_options) const;
+                                          const DBOptions& db_options) const;
 
   // OptimizeForManifestWrite will create a new FileOptions object that is a
   // copy of the FileOptions in the parameters, but is optimized for writing
@@ -686,6 +684,10 @@ class FileSystem : public Customizable {
     return IOStatus::OK();
   }
 
+  // Indicates to upper layers whether the FileSystem supports/uses async IO
+  // or not
+  virtual bool use_async_io() { return true; }
+
   // If you're adding methods here, remember to add them to EnvWrapper too.
 
  private:
@@ -1328,8 +1330,7 @@ class FileSystemWrapper : public FileSystem {
   FileSystem* target() const { return target_.get(); }
 
   // The following text is boilerplate that forwards all methods to target()
-  IOStatus NewSequentialFile(const std::string& f,
-                             const FileOptions& file_opts,
+  IOStatus NewSequentialFile(const std::string& f, const FileOptions& file_opts,
                              std::unique_ptr<FSSequentialFile>* r,
                              IODebugContext* dbg) override {
     return target_->NewSequentialFile(f, file_opts, r, dbg);
@@ -1356,8 +1357,7 @@ class FileSystemWrapper : public FileSystem {
                              const FileOptions& file_opts,
                              std::unique_ptr<FSWritableFile>* r,
                              IODebugContext* dbg) override {
-    return target_->ReuseWritableFile(fname, old_fname, file_opts, r,
-                                      dbg);
+    return target_->ReuseWritableFile(fname, old_fname, file_opts, r, dbg);
   }
   IOStatus NewRandomRWFile(const std::string& fname,
                            const FileOptions& file_opts,
@@ -1474,7 +1474,7 @@ class FileSystemWrapper : public FileSystem {
   }
 
   FileOptions OptimizeForLogRead(
-                  const FileOptions& file_options) const override {
+      const FileOptions& file_options) const override {
     return target_->OptimizeForLogRead(file_options);
   }
   FileOptions OptimizeForManifestRead(
@@ -1482,7 +1482,7 @@ class FileSystemWrapper : public FileSystem {
     return target_->OptimizeForManifestRead(file_options);
   }
   FileOptions OptimizeForLogWrite(const FileOptions& file_options,
-                                 const DBOptions& db_options) const override {
+                                  const DBOptions& db_options) const override {
     return target_->OptimizeForLogWrite(file_options, db_options);
   }
   FileOptions OptimizeForManifestWrite(
@@ -1516,10 +1516,8 @@ class FileSystemWrapper : public FileSystem {
 
   const Customizable* Inner() const override { return target_.get(); }
   Status PrepareOptions(const ConfigOptions& options) override;
-#ifndef ROCKSDB_LITE
   std::string SerializeOptions(const ConfigOptions& config_options,
                                const std::string& header) const override;
-#endif  // ROCKSDB_LITE
 
   virtual IOStatus Poll(std::vector<void*>& io_handles,
                         size_t min_completions) override {
@@ -1530,6 +1528,8 @@ class FileSystemWrapper : public FileSystem {
     return target_->AbortIO(io_handles);
   }
 
+  virtual bool use_async_io() override { return target_->use_async_io(); }
+
  protected:
   std::shared_ptr<FileSystem> target_;
 };
