@@ -13,20 +13,20 @@
 
 #pragma once
 
-#include <stdint.h>
 #include <cstddef>
+#include <cstdint>
 #include <map>
 #include <string>
 #include <utility>
 #include <vector>
 
-#if !defined(ROCKSDB_LITE) && \
-    !defined(NROCKSDB_THREAD_STATUS) && \
-    defined(ROCKSDB_SUPPORT_THREAD_LOCAL)
+#include "rocksdb/rocksdb_namespace.h"
+
+#if !defined(NROCKSDB_THREAD_STATUS)
 #define ROCKSDB_USING_THREAD_STATUS
 #endif
 
-namespace rocksdb {
+namespace ROCKSDB_NAMESPACE {
 
 // TODO(yhchiang): remove this function once c++14 is available
 //                 as std::max will be able to cover this.
@@ -38,14 +38,14 @@ struct constexpr_max {
 
 // A structure that describes the current status of a thread.
 // The status of active threads can be fetched using
-// rocksdb::GetThreadList().
+// ROCKSDB_NAMESPACE::GetThreadList().
 struct ThreadStatus {
   // The type of a thread.
   enum ThreadType : int {
     HIGH_PRIORITY = 0,  // RocksDB BG thread in high-pri thread pool
-    LOW_PRIORITY,  // RocksDB BG thread in low-pri thread pool
-    USER,  // User thread (Non-RocksDB BG thread)
-    BOTTOM_PRIORITY,  // RocksDB BG thread in bottom-pri thread pool
+    LOW_PRIORITY,       // RocksDB BG thread in low-pri thread pool
+    USER,               // User thread (Non-RocksDB BG thread)
+    BOTTOM_PRIORITY,    // RocksDB BG thread in bottom-pri thread pool
     NUM_THREAD_TYPES
   };
 
@@ -56,6 +56,7 @@ struct ThreadStatus {
     OP_UNKNOWN = 0,
     OP_COMPACTION,
     OP_FLUSH,
+    OP_DBOPEN,
     NUM_OP_TYPES
   };
 
@@ -105,22 +106,20 @@ struct ThreadStatus {
     NUM_STATE_TYPES
   };
 
-  ThreadStatus(const uint64_t _id,
-               const ThreadType _thread_type,
-               const std::string& _db_name,
-               const std::string& _cf_name,
+  ThreadStatus(const uint64_t _id, const ThreadType _thread_type,
+               const std::string& _db_name, const std::string& _cf_name,
                const OperationType _operation_type,
                const uint64_t _op_elapsed_micros,
                const OperationStage _operation_stage,
-               const uint64_t _op_props[],
-               const StateType _state_type) :
-      thread_id(_id), thread_type(_thread_type),
-      db_name(_db_name),
-      cf_name(_cf_name),
-      operation_type(_operation_type),
-      op_elapsed_micros(_op_elapsed_micros),
-      operation_stage(_operation_stage),
-      state_type(_state_type) {
+               const uint64_t _op_props[], const StateType _state_type)
+      : thread_id(_id),
+        thread_type(_thread_type),
+        db_name(_db_name),
+        cf_name(_cf_name),
+        operation_type(_operation_type),
+        op_elapsed_micros(_op_elapsed_micros),
+        operation_stage(_operation_stage),
+        state_type(_state_type) {
     for (int i = 0; i < kNumOperationProperties; ++i) {
       op_properties[i] = _op_props[i];
     }
@@ -172,23 +171,20 @@ struct ThreadStatus {
   static const std::string MicrosToString(uint64_t op_elapsed_time);
 
   // Obtain a human-readable string describing the specified operation stage.
-  static const std::string& GetOperationStageName(
-      OperationStage stage);
+  static const std::string& GetOperationStageName(OperationStage stage);
 
   // Obtain the name of the "i"th operation property of the
   // specified operation.
-  static const std::string& GetOperationPropertyName(
-      OperationType op_type, int i);
+  static const std::string& GetOperationPropertyName(OperationType op_type,
+                                                     int i);
 
   // Translate the "i"th property of the specified operation given
   // a property value.
-  static std::map<std::string, uint64_t>
-      InterpretOperationProperties(
-          OperationType op_type, const uint64_t* op_properties);
+  static std::map<std::string, uint64_t> InterpretOperationProperties(
+      OperationType op_type, const uint64_t* op_properties);
 
   // Obtain the name of a state given its type.
   static const std::string& GetStateName(StateType state_type);
 };
 
-
-}  // namespace rocksdb
+}  // namespace ROCKSDB_NAMESPACE
