@@ -5,7 +5,6 @@
 
 #pragma once
 
-#ifndef ROCKSDB_LITE
 
 #include <limits>
 #include <string>
@@ -627,7 +626,7 @@ class Transaction {
     PREPARED = 2,
     AWAITING_COMMIT = 3,
     COMMITTED = 4,
-    COMMITED = COMMITTED, // old misspelled name
+    COMMITED = COMMITTED,  // old misspelled name
     AWAITING_ROLLBACK = 5,
     ROLLEDBACK = 6,
     LOCKS_STOLEN = 7,
@@ -682,5 +681,3 @@ class Transaction {
 };
 
 }  // namespace ROCKSDB_NAMESPACE
-
-#endif  // ROCKSDB_LITE
