@@ -303,7 +303,7 @@ static void DBPut(benchmark::State& state) {
 
   if (state.thread_index() == 0) {
     auto db_full = static_cast_with_check<DBImpl>(db.get());
-    Status s = db_full->WaitForCompact(true);
+    Status s = db_full->WaitForCompact();
     if (!s.ok()) {
       state.SkipWithError(s.ToString().c_str());
       return;
@@ -410,7 +410,7 @@ static void ManualCompaction(benchmark::State& state) {
 
   if (state.thread_index() == 0) {
     auto db_full = static_cast_with_check<DBImpl>(db.get());
-    s = db_full->WaitForCompact(true);
+    s = db_full->WaitForCompact();
     if (!s.ok()) {
       state.SkipWithError(s.ToString().c_str());
       return;
@@ -436,6 +436,8 @@ static void ManualCompaction(benchmark::State& state) {
           static_cast<double>(get_perf_context()->block_read_count);
       state.counters["block_read_time"] =
           static_cast<double>(get_perf_context()->block_read_time);
+      state.counters["block_read_cpu_time"] =
+          static_cast<double>(get_perf_context()->block_read_cpu_time);
       state.counters["block_checksum_time"] =
           static_cast<double>(get_perf_context()->block_checksum_time);
       state.counters["new_table_block_iter_nanos"] =
@@ -508,7 +510,7 @@ static void ManualFlush(benchmark::State& state) {
 
   if (state.thread_index() == 0) {
     auto db_full = static_cast_with_check<DBImpl>(db.get());
-    Status s = db_full->WaitForCompact(true);
+    Status s = db_full->WaitForCompact();
     if (!s.ok()) {
       state.SkipWithError(s.ToString().c_str());
       return;
@@ -592,7 +594,7 @@ static void DBGet(benchmark::State& state) {
     }
 
     auto db_full = static_cast_with_check<DBImpl>(db.get());
-    s = db_full->WaitForCompact(true);
+    s = db_full->WaitForCompact();
     if (!s.ok()) {
       state.SkipWithError(s.ToString().c_str());
       return;
@@ -705,7 +707,7 @@ static void SimpleGetWithPerfContext(benchmark::State& state) {
       }
     }
     auto db_full = static_cast_with_check<DBImpl>(db.get());
-    s = db_full->WaitForCompact(true);
+    s = db_full->WaitForCompact();
     if (!s.ok()) {
       state.SkipWithError(s.ToString().c_str());
       return;
@@ -721,6 +723,7 @@ static void SimpleGetWithPerfContext(benchmark::State& state) {
   size_t not_found = 0;
   uint64_t user_key_comparison_count = 0;
   uint64_t block_read_time = 0;
+  uint64_t block_read_cpu_time = 0;
   uint64_t block_checksum_time = 0;
   uint64_t get_snapshot_time = 0;
   uint64_t get_post_process_time = 0;
@@ -740,6 +743,7 @@ static void SimpleGetWithPerfContext(benchmark::State& state) {
     }
     user_key_comparison_count += get_perf_context()->user_key_comparison_count;
     block_read_time += get_perf_context()->block_read_time;
+    block_read_cpu_time += get_perf_context()->block_read_cpu_time;
     block_checksum_time += get_perf_context()->block_checksum_time;
     get_snapshot_time += get_perf_context()->get_snapshot_time;
     get_post_process_time += get_perf_context()->get_post_process_time;
@@ -760,6 +764,9 @@ static void SimpleGetWithPerfContext(benchmark::State& state) {
                          benchmark::Counter::kAvgIterations);
   state.counters["block_read_time"] = benchmark::Counter(
       static_cast<double>(block_read_time), benchmark::Counter::kAvgIterations);
+  state.counters["block_read_cpu_time"] =
+      benchmark::Counter(static_cast<double>(block_read_cpu_time),
+                         benchmark::Counter::kAvgIterations);
   state.counters["block_checksum_time"] =
       benchmark::Counter(static_cast<double>(block_checksum_time),
                          benchmark::Counter::kAvgIterations);
@@ -841,6 +848,9 @@ static void DBGetMergeOperandsInMemtable(benchmark::State& state) {
     if (num_value_operands != static_cast<int>(kNumEntriesPerKey)) {
       state.SkipWithError("Unexpected number of merge operands found for key");
     }
+    for (auto& value_operand : value_operands) {
+      value_operand.Reset();
+    }
   }
 
   if (state.thread_index() == 0) {
@@ -938,6 +948,9 @@ static void DBGetMergeOperandsInSstFile(benchmark::State& state) {
     if (num_value_operands != static_cast<int>(kNumEntriesPerKey)) {
       state.SkipWithError("Unexpected number of merge operands found for key");
     }
+    for (auto& value_operand : value_operands) {
+      value_operand.Reset();
+    }
   }
 
   if (state.thread_index() == 0) {
@@ -1102,28 +1115,26 @@ static void IteratorSeek(benchmark::State& state) {
     }
 
     auto db_full = static_cast_with_check<DBImpl>(db.get());
-    s = db_full->WaitForCompact(true);
+    s = db_full->WaitForCompact();
     if (!s.ok()) {
       state.SkipWithError(s.ToString().c_str());
       return;
     }
   }
 
-  {
+  for (auto _ : state) {
     std::unique_ptr<Iterator> iter{nullptr};
-    for (auto _ : state) {
-      state.PauseTiming();
-      if (!iter) {
-        iter.reset(db->NewIterator(ReadOptions()));
-      }
-      Slice key = negative_query ? kg.NextNonExist() : kg.Next();
-      if (!iter->status().ok()) {
-        state.SkipWithError(iter->status().ToString().c_str());
-        return;
-      }
-      state.ResumeTiming();
-      iter->Seek(key);
+    state.PauseTiming();
+    if (!iter) {
+      iter.reset(db->NewIterator(ReadOptions()));
     }
+    Slice key = negative_query ? kg.NextNonExist() : kg.Next();
+    if (!iter->status().ok()) {
+      state.SkipWithError(iter->status().ToString().c_str());
+      return;
+    }
+    state.ResumeTiming();
+    iter->Seek(key);
   }
 
   if (state.thread_index() == 0) {
@@ -1195,29 +1206,27 @@ static void IteratorNext(benchmark::State& state) {
     }
 
     auto db_full = static_cast_with_check<DBImpl>(db.get());
-    s = db_full->WaitForCompact(true);
+    s = db_full->WaitForCompact();
     if (!s.ok()) {
       state.SkipWithError(s.ToString().c_str());
       return;
     }
   }
 
-  {
+  for (auto _ : state) {
     std::unique_ptr<Iterator> iter{nullptr};
-    for (auto _ : state) {
-      state.PauseTiming();
-      if (!iter) {
-        iter.reset(db->NewIterator(ReadOptions()));
-      }
-      while (!iter->Valid()) {
-        iter->Seek(kg.Next());
-        if (!iter->status().ok()) {
-          state.SkipWithError(iter->status().ToString().c_str());
-        }
-      }
-      state.ResumeTiming();
-      iter->Next();
+    state.PauseTiming();
+    if (!iter) {
+      iter.reset(db->NewIterator(ReadOptions()));
     }
+    while (!iter->Valid()) {
+      iter->Seek(kg.Next());
+      if (!iter->status().ok()) {
+        state.SkipWithError(iter->status().ToString().c_str());
+      }
+    }
+    state.ResumeTiming();
+    iter->Next();
   }
 
   if (state.thread_index() == 0) {
@@ -1261,7 +1270,7 @@ static void IteratorNextWithPerfContext(benchmark::State& state) {
       }
     }
     auto db_full = static_cast_with_check<DBImpl>(db.get());
-    Status s = db_full->WaitForCompact(true);
+    Status s = db_full->WaitForCompact();
     if (!s.ok()) {
       state.SkipWithError(s.ToString().c_str());
       return;
@@ -1281,31 +1290,27 @@ static void IteratorNextWithPerfContext(benchmark::State& state) {
   SetPerfLevel(kEnableTime);
   get_perf_context()->EnablePerLevelPerfContext();
 
-  {
+  for (auto _ : state) {
     std::unique_ptr<Iterator> iter{nullptr};
-    for (auto _ : state) {
-      state.PauseTiming();
-      if (!iter) {
-        iter.reset(db->NewIterator(ReadOptions()));
-      }
-      while (!iter->Valid()) {
-        iter->Seek(kg.Next());
-        if (!iter->status().ok()) {
-          state.SkipWithError(iter->status().ToString().c_str());
-        }
-      }
-      get_perf_context()->Reset();
-      state.ResumeTiming();
-
-      iter->Next();
-      user_key_comparison_count +=
-          get_perf_context()->user_key_comparison_count;
-      internal_key_skipped_count +=
-          get_perf_context()->internal_key_skipped_count;
-      find_next_user_entry_time +=
-          get_perf_context()->find_next_user_entry_time;
-      iter_next_cpu_nanos += get_perf_context()->iter_next_cpu_nanos;
+    state.PauseTiming();
+    if (!iter) {
+      iter.reset(db->NewIterator(ReadOptions()));
     }
+    while (!iter->Valid()) {
+      iter->Seek(kg.Next());
+      if (!iter->status().ok()) {
+        state.SkipWithError(iter->status().ToString().c_str());
+      }
+    }
+    get_perf_context()->Reset();
+    state.ResumeTiming();
+
+    iter->Next();
+    user_key_comparison_count += get_perf_context()->user_key_comparison_count;
+    internal_key_skipped_count +=
+        get_perf_context()->internal_key_skipped_count;
+    find_next_user_entry_time += get_perf_context()->find_next_user_entry_time;
+    iter_next_cpu_nanos += get_perf_context()->iter_next_cpu_nanos;
   }
 
   state.counters["user_key_comparison_count"] =
@@ -1363,29 +1368,27 @@ static void IteratorPrev(benchmark::State& state) {
     }
 
     auto db_full = static_cast_with_check<DBImpl>(db.get());
-    s = db_full->WaitForCompact(true);
+    s = db_full->WaitForCompact();
     if (!s.ok()) {
       state.SkipWithError(s.ToString().c_str());
       return;
     }
   }
 
-  {
+  for (auto _ : state) {
     std::unique_ptr<Iterator> iter{nullptr};
-    for (auto _ : state) {
-      state.PauseTiming();
-      if (!iter) {
-        iter.reset(db->NewIterator(ReadOptions()));
-      }
-      while (!iter->Valid()) {
-        iter->Seek(kg.Next());
-        if (!iter->status().ok()) {
-          state.SkipWithError(iter->status().ToString().c_str());
-        }
-      }
-      state.ResumeTiming();
-      iter->Prev();
+    state.PauseTiming();
+    if (!iter) {
+      iter.reset(db->NewIterator(ReadOptions()));
     }
+    while (!iter->Valid()) {
+      iter->Seek(kg.Next());
+      if (!iter->status().ok()) {
+        state.SkipWithError(iter->status().ToString().c_str());
+      }
+    }
+    state.ResumeTiming();
+    iter->Prev();
   }
 
   if (state.thread_index() == 0) {
@@ -1457,26 +1460,24 @@ static void PrefixSeek(benchmark::State& state) {
     }
 
     auto db_full = static_cast_with_check<DBImpl>(db.get());
-    s = db_full->WaitForCompact(true);
+    s = db_full->WaitForCompact();
     if (!s.ok()) {
       state.SkipWithError(s.ToString().c_str());
       return;
     }
   }
 
-  {
+  for (auto _ : state) {
     std::unique_ptr<Iterator> iter{nullptr};
-    for (auto _ : state) {
-      state.PauseTiming();
-      if (!iter) {
-        iter.reset(db->NewIterator(ReadOptions()));
-      }
-      state.ResumeTiming();
-      iter->Seek(kg.NextPrefix());
-      if (!iter->status().ok()) {
-        state.SkipWithError(iter->status().ToString().c_str());
-        return;
-      }
+    state.PauseTiming();
+    if (!iter) {
+      iter.reset(db->NewIterator(ReadOptions()));
+    }
+    state.ResumeTiming();
+    iter->Seek(kg.NextPrefix());
+    if (!iter->status().ok()) {
+      state.SkipWithError(iter->status().ToString().c_str());
+      return;
     }
   }
 
@@ -1547,7 +1548,8 @@ static void RandomAccessFileReaderRead(benchmark::State& state) {
                                        : Temperature::kCold;
     readers.emplace_back(new RandomAccessFileReader(
         std::move(f), fname, env->GetSystemClock().get(), nullptr, statistics,
-        0, nullptr, nullptr, {}, temperature, rand_num == 1));
+        Histograms::HISTOGRAM_ENUM_MAX, nullptr, nullptr, {}, temperature,
+        rand_num == 1));
   }
 
   IOOptions io_options;
