@@ -21,13 +21,15 @@
 #include <unordered_map>
 #include <utility>
 
+#include "block_cache.h"
 #include "cache/cache_entry_roles.h"
+#include "cache/cache_helpers.h"
 #include "cache/cache_key.h"
 #include "cache/cache_reservation_manager.h"
 #include "db/dbformat.h"
 #include "index_builder.h"
 #include "logging/logging.h"
-#include "memory/memory_allocator.h"
+#include "memory/memory_allocator_impl.h"
 #include "rocksdb/cache.h"
 #include "rocksdb/comparator.h"
 #include "rocksdb/env.h"
@@ -40,7 +42,6 @@
 #include "table/block_based/block_based_table_factory.h"
 #include "table/block_based/block_based_table_reader.h"
 #include "table/block_based/block_builder.h"
-#include "table/block_based/block_like_traits.h"
 #include "table/block_based/filter_block.h"
 #include "table/block_based/filter_policy_internal.h"
 #include "table/block_based/full_filter_block.h"
@@ -59,7 +60,6 @@ namespace ROCKSDB_NAMESPACE {
 extern const std::string kHashIndexPrefixesBlock;
 extern const std::string kHashIndexPrefixesMetadataBlock;
 
-
 // Without anonymous namespace here, we fail the warning -Wmissing-prototypes
 namespace {
 
@@ -104,9 +104,12 @@ FilterBlockBuilder* CreateFilterBlockBuilder(
   }
 }
 
-bool GoodCompressionRatio(size_t compressed_size, size_t uncomp_size) {
-  // Check to see if compressed less than 12.5%
-  return compressed_size < uncomp_size - (uncomp_size / 8u);
+bool GoodCompressionRatio(size_t compressed_size, size_t uncomp_size,
+                          int max_compressed_bytes_per_kb) {
+  // For efficiency, avoid floating point and division
+  return compressed_size <=
+         (static_cast<uint64_t>(max_compressed_bytes_per_kb) * uncomp_size) >>
+         10;
 }
 
 }  // namespace
@@ -114,7 +117,7 @@ bool GoodCompressionRatio(size_t compressed_size, size_t uncomp_size) {
 // format_version is the block format as defined in include/rocksdb/table.h
 Slice CompressBlock(const Slice& uncompressed_data, const CompressionInfo& info,
                     CompressionType* type, uint32_t format_version,
-                    bool do_sample, std::string* compressed_output,
+                    bool allow_sample, std::string* compressed_output,
                     std::string* sampled_output_fast,
                     std::string* sampled_output_slow) {
   assert(type);
@@ -126,7 +129,7 @@ Slice CompressBlock(const Slice& uncompressed_data, const CompressionInfo& info,
   // The users can use these stats to decide if it is worthwhile
   // enabling compression and they also get a hint about which
   // compression algorithm wil be beneficial.
-  if (do_sample && info.SampleForCompression() &&
+  if (allow_sample && info.SampleForCompression() &&
       Random::GetTLSInstance()->OneIn(
           static_cast<int>(info.SampleForCompression()))) {
     // Sampling with a fast compression algorithm
@@ -159,7 +162,8 @@ Slice CompressBlock(const Slice& uncompressed_data, const CompressionInfo& info,
     }
   }
 
-  if (info.type() == kNoCompression) {
+  int max_compressed_bytes_per_kb = info.options().max_compressed_bytes_per_kb;
+  if (info.type() == kNoCompression || max_compressed_bytes_per_kb <= 0) {
     *type = kNoCompression;
     return uncompressed_data;
   }
@@ -175,8 +179,8 @@ Slice CompressBlock(const Slice& uncompressed_data, const CompressionInfo& info,
 
   // Check the compression ratio; if it's not good enough, just fall back to
   // uncompressed
-  if (!GoodCompressionRatio(compressed_output->size(),
-                            uncompressed_data.size())) {
+  if (!GoodCompressionRatio(compressed_output->size(), uncompressed_data.size(),
+                            max_compressed_bytes_per_kb)) {
     *type = kNoCompression;
     return uncompressed_data;
   }
@@ -335,6 +339,11 @@ struct BlockBasedTableBuilder::Rep {
   std::vector<std::unique_ptr<IntTblPropCollector>> table_properties_collectors;
 
   std::unique_ptr<ParallelCompressionRep> pc_rep;
+  BlockCreateContext create_context;
+
+  // The size of the "tail" part of a SST file. "Tail" refers to
+  // all blocks after data blocks till the end of the SST file.
+  uint64_t tail_size;
 
   uint64_t get_offset() { return offset.load(std::memory_order_relaxed); }
   void set_offset(uint64_t o) { offset.store(o, std::memory_order_relaxed); }
@@ -443,6 +452,15 @@ struct BlockBasedTableBuilder::Rep {
         flush_block_policy(
             table_options.flush_block_policy_factory->NewFlushBlockPolicy(
                 table_options, data_block)),
+        create_context(&table_options, ioptions.stats,
+                       compression_type == kZSTD ||
+                           compression_type == kZSTDNotFinalCompression,
+                       tbo.moptions.block_protection_bytes_per_key,
+                       tbo.internal_comparator.user_comparator(),
+                       !use_delta_encoding_for_index_values,
+                       table_opt.index_type ==
+                           BlockBasedTableOptions::kBinarySearchWithFirstKey),
+        tail_size(0),
         status_ok(true),
         io_status_ok(true) {
     if (tbo.target_file_size == 0) {
@@ -1011,7 +1029,8 @@ void BlockBasedTableBuilder::Add(const Slice& key, const Slice& value) {
   r->props.num_entries++;
   r->props.raw_key_size += key.size();
   r->props.raw_value_size += value.size();
-  if (value_type == kTypeDeletion || value_type == kTypeSingleDeletion) {
+  if (value_type == kTypeDeletion || value_type == kTypeSingleDeletion ||
+      value_type == kTypeDeletionWithTimestamp) {
     r->props.num_deletions++;
   } else if (value_type == kTypeRangeDeletion) {
     r->props.num_deletions++;
@@ -1103,25 +1122,17 @@ void BlockBasedTableBuilder::CompressAndVerifyBlock(
     const CompressionContext& compression_ctx, UncompressionContext* verify_ctx,
     std::string* compressed_output, Slice* block_contents,
     CompressionType* type, Status* out_status) {
-  // File format contains a sequence of blocks where each block has:
-  //    block_data: uint8[n]
-  //    type: uint8
-  //    crc: uint32
   Rep* r = rep_;
   bool is_status_ok = ok();
   if (!r->IsParallelCompressionEnabled()) {
     assert(is_status_ok);
   }
 
-  *type = r->compression_type;
-  uint64_t sample_for_compression = r->sample_for_compression;
-  bool abort_compression = false;
-
-  StopWatchNano timer(
-      r->ioptions.clock,
-      ShouldReportDetailedTime(r->ioptions.env, r->ioptions.stats));
-
   if (is_status_ok && uncompressed_block_data.size() < kCompressionSizeLimit) {
+    StopWatchNano timer(
+        r->ioptions.clock,
+        ShouldReportDetailedTime(r->ioptions.env, r->ioptions.stats));
+
     if (is_data_block) {
       r->compressible_input_data_bytes.fetch_add(uncompressed_block_data.size(),
                                                  std::memory_order_relaxed);
@@ -1134,14 +1145,14 @@ void BlockBasedTableBuilder::CompressAndVerifyBlock(
     }
     assert(compression_dict != nullptr);
     CompressionInfo compression_info(r->compression_opts, compression_ctx,
-                                     *compression_dict, *type,
-                                     sample_for_compression);
+                                     *compression_dict, r->compression_type,
+                                     r->sample_for_compression);
 
     std::string sampled_output_fast;
     std::string sampled_output_slow;
     *block_contents = CompressBlock(
         uncompressed_block_data, compression_info, type,
-        r->table_options.format_version, is_data_block /* do_sample */,
+        r->table_options.format_version, is_data_block /* allow_sample */,
         compressed_output, &sampled_output_fast, &sampled_output_slow);
 
     if (sampled_output_slow.size() > 0 || sampled_output_fast.size() > 0) {
@@ -1174,35 +1185,38 @@ void BlockBasedTableBuilder::CompressAndVerifyBlock(
       BlockContents contents;
       UncompressionInfo uncompression_info(*verify_ctx, *verify_dict,
                                            r->compression_type);
-      Status stat = UncompressBlockData(
+      Status uncompress_status = UncompressBlockData(
           uncompression_info, block_contents->data(), block_contents->size(),
           &contents, r->table_options.format_version, r->ioptions);
 
-      if (stat.ok()) {
-        bool compressed_ok =
-            contents.data.compare(uncompressed_block_data) == 0;
-        if (!compressed_ok) {
+      if (uncompress_status.ok()) {
+        bool data_match = contents.data.compare(uncompressed_block_data) == 0;
+        if (!data_match) {
           // The result of the compression was invalid. abort.
-          abort_compression = true;
           const char* const msg =
               "Decompressed block did not match pre-compression block";
           ROCKS_LOG_ERROR(r->ioptions.logger, "%s", msg);
           *out_status = Status::Corruption(msg);
+          *type = kNoCompression;
         }
       } else {
         // Decompression reported an error. abort.
         *out_status = Status::Corruption(std::string("Could not decompress: ") +
-                                         stat.getState());
-        abort_compression = true;
+                                         uncompress_status.getState());
+        *type = kNoCompression;
       }
     }
+    if (timer.IsStarted()) {
+      RecordTimeToHistogram(r->ioptions.stats, COMPRESSION_TIMES_NANOS,
+                            timer.ElapsedNanos());
+    }
   } else {
-    // Block is too big to be compressed.
+    // Status is not OK, or block is too big to be compressed.
     if (is_data_block) {
       r->uncompressible_input_data_bytes.fetch_add(
           uncompressed_block_data.size(), std::memory_order_relaxed);
     }
-    abort_compression = true;
+    *type = kNoCompression;
   }
   if (is_data_block) {
     r->uncompressible_input_data_bytes.fetch_add(kBlockTrailerSize,
@@ -1211,26 +1225,32 @@ void BlockBasedTableBuilder::CompressAndVerifyBlock(
 
   // Abort compression if the block is too big, or did not pass
   // verification.
-  if (abort_compression) {
-    RecordTick(r->ioptions.stats, NUMBER_BLOCK_NOT_COMPRESSED);
-    *type = kNoCompression;
+  if (*type == kNoCompression) {
     *block_contents = uncompressed_block_data;
-  } else if (*type != kNoCompression) {
-    if (ShouldReportDetailedTime(r->ioptions.env, r->ioptions.stats)) {
-      RecordTimeToHistogram(r->ioptions.stats, COMPRESSION_TIMES_NANOS,
-                            timer.ElapsedNanos());
-    }
-    RecordInHistogram(r->ioptions.stats, BYTES_COMPRESSED,
-                      uncompressed_block_data.size());
+    bool compression_attempted = !compressed_output->empty();
+    RecordTick(r->ioptions.stats, compression_attempted
+                                      ? NUMBER_BLOCK_COMPRESSION_REJECTED
+                                      : NUMBER_BLOCK_COMPRESSION_BYPASSED);
+    RecordTick(r->ioptions.stats,
+               compression_attempted ? BYTES_COMPRESSION_REJECTED
+                                     : BYTES_COMPRESSION_BYPASSED,
+               uncompressed_block_data.size());
+  } else {
     RecordTick(r->ioptions.stats, NUMBER_BLOCK_COMPRESSED);
-  } else if (*type != r->compression_type) {
-    RecordTick(r->ioptions.stats, NUMBER_BLOCK_NOT_COMPRESSED);
+    RecordTick(r->ioptions.stats, BYTES_COMPRESSED_FROM,
+               uncompressed_block_data.size());
+    RecordTick(r->ioptions.stats, BYTES_COMPRESSED_TO,
+               compressed_output->size());
   }
 }
 
 void BlockBasedTableBuilder::WriteMaybeCompressedBlock(
-    const Slice& block_contents, CompressionType type, BlockHandle* handle,
+    const Slice& block_contents, CompressionType comp_type, BlockHandle* handle,
     BlockType block_type, const Slice* uncompressed_block_data) {
+  // File format contains a sequence of blocks where each block has:
+  //    block_data: uint8[n]
+  //    compression_type: uint8
+  //    checksum: uint32
   Rep* r = rep_;
   bool is_data_block = block_type == BlockType::kData;
   // Old, misleading name of this function: WriteRawBlock
@@ -1239,6 +1259,10 @@ void BlockBasedTableBuilder::WriteMaybeCompressedBlock(
   handle->set_size(block_contents.size());
   assert(status().ok());
   assert(io_status().ok());
+  if (uncompressed_block_data == nullptr) {
+    uncompressed_block_data = &block_contents;
+    assert(comp_type == kNoCompression);
+  }
 
   {
     IOStatus io_s = r->file->Append(block_contents);
@@ -1249,10 +1273,10 @@ void BlockBasedTableBuilder::WriteMaybeCompressedBlock(
   }
 
   std::array<char, kBlockTrailerSize> trailer;
-  trailer[0] = type;
+  trailer[0] = comp_type;
   uint32_t checksum = ComputeBuiltinChecksumWithLastByte(
       r->table_options.checksum, block_contents.data(), block_contents.size(),
-      /*last_byte*/ type);
+      /*last_byte*/ comp_type);
 
   if (block_type == BlockType::kFilter) {
     Status s = r->filter_builder->MaybePostVerifyFilter(block_contents);
@@ -1275,7 +1299,6 @@ void BlockBasedTableBuilder::WriteMaybeCompressedBlock(
   }
 
   {
-    Status s = Status::OK();
     bool warm_cache;
     switch (r->table_options.prepopulate_block_cache) {
       case BlockBasedTableOptions::PrepopulateBlockCache::kFlushOnly:
@@ -1290,22 +1313,13 @@ void BlockBasedTableBuilder::WriteMaybeCompressedBlock(
         warm_cache = false;
     }
     if (warm_cache) {
-      if (type == kNoCompression) {
-        s = InsertBlockInCacheHelper(block_contents, handle, block_type);
-      } else if (uncompressed_block_data != nullptr) {
-        s = InsertBlockInCacheHelper(*uncompressed_block_data, handle,
-                                     block_type);
-      }
+      Status s = InsertBlockInCacheHelper(*uncompressed_block_data, handle,
+                                          block_type);
       if (!s.ok()) {
         r->SetStatus(s);
         return;
       }
     }
-    s = InsertBlockInCompressedCache(block_contents, type, handle);
-    if (!s.ok()) {
-      r->SetStatus(s);
-      return;
-    }
   }
 
   r->set_offset(r->get_offset() + block_contents.size() + kBlockTrailerSize);
@@ -1417,118 +1431,22 @@ IOStatus BlockBasedTableBuilder::io_status() const {
   return rep_->GetIOStatus();
 }
 
-namespace {
-// Delete the entry resided in the cache.
-template <class Entry>
-void DeleteEntryCached(const Slice& /*key*/, void* value) {
-  auto entry = reinterpret_cast<Entry*>(value);
-  delete entry;
-}
-}  // namespace
-
-//
-// Make a copy of the block contents and insert into compressed block cache
-//
-Status BlockBasedTableBuilder::InsertBlockInCompressedCache(
-    const Slice& block_contents, const CompressionType type,
-    const BlockHandle* handle) {
-  Rep* r = rep_;
-  Cache* block_cache_compressed = r->table_options.block_cache_compressed.get();
-  Status s;
-  if (type != kNoCompression && block_cache_compressed != nullptr) {
-    size_t size = block_contents.size();
-
-    auto ubuf =
-        AllocateBlock(size + 1, block_cache_compressed->memory_allocator());
-    memcpy(ubuf.get(), block_contents.data(), size);
-    ubuf[size] = type;
-
-    BlockContents* block_contents_to_cache =
-        new BlockContents(std::move(ubuf), size);
-#ifndef NDEBUG
-    block_contents_to_cache->has_trailer = true;
-#endif  // NDEBUG
-
-    CacheKey key = BlockBasedTable::GetCacheKey(rep_->base_cache_key, *handle);
-
-    s = block_cache_compressed->Insert(
-        key.AsSlice(), block_contents_to_cache,
-        block_contents_to_cache->ApproximateMemoryUsage(),
-        &DeleteEntryCached<BlockContents>);
-    if (s.ok()) {
-      RecordTick(rep_->ioptions.stats, BLOCK_CACHE_COMPRESSED_ADD);
-    } else {
-      RecordTick(rep_->ioptions.stats, BLOCK_CACHE_COMPRESSED_ADD_FAILURES);
-    }
-    // Invalidate OS cache.
-    r->file->InvalidateCache(static_cast<size_t>(r->get_offset()), size)
-        .PermitUncheckedError();
-  }
-  return s;
-}
-
 Status BlockBasedTableBuilder::InsertBlockInCacheHelper(
     const Slice& block_contents, const BlockHandle* handle,
     BlockType block_type) {
-  Status s;
-  switch (block_type) {
-    case BlockType::kData:
-    case BlockType::kIndex:
-    case BlockType::kFilterPartitionIndex:
-      s = InsertBlockInCache<Block>(block_contents, handle, block_type);
-      break;
-    case BlockType::kFilter:
-      s = InsertBlockInCache<ParsedFullFilterBlock>(block_contents, handle,
-                                                    block_type);
-      break;
-    case BlockType::kCompressionDictionary:
-      s = InsertBlockInCache<UncompressionDict>(block_contents, handle,
-                                                block_type);
-      break;
-    default:
-      // no-op / not cached
-      break;
-  }
-  return s;
-}
 
-template <typename TBlocklike>
-Status BlockBasedTableBuilder::InsertBlockInCache(const Slice& block_contents,
-                                                  const BlockHandle* handle,
-                                                  BlockType block_type) {
-  // Uncompressed regular block cache
   Cache* block_cache = rep_->table_options.block_cache.get();
   Status s;
-  if (block_cache != nullptr) {
-    size_t size = block_contents.size();
-    auto buf = AllocateBlock(size, block_cache->memory_allocator());
-    memcpy(buf.get(), block_contents.data(), size);
-    BlockContents results(std::move(buf), size);
-
+  auto helper =
+      GetCacheItemHelper(block_type, rep_->ioptions.lowest_used_cache_tier);
+  if (block_cache && helper && helper->create_cb) {
     CacheKey key = BlockBasedTable::GetCacheKey(rep_->base_cache_key, *handle);
-
-    const size_t read_amp_bytes_per_bit =
-        rep_->table_options.read_amp_bytes_per_bit;
-
-    // TODO akanksha:: Dedup below code by calling
-    // BlockBasedTable::PutDataBlockToCache.
-    std::unique_ptr<TBlocklike> block_holder(
-        BlocklikeTraits<TBlocklike>::Create(
-            std::move(results), read_amp_bytes_per_bit,
-            rep_->ioptions.statistics.get(),
-            false /*rep_->blocks_definitely_zstd_compressed*/,
-            rep_->table_options.filter_policy.get()));
-
-    assert(block_holder->own_bytes());
-    size_t charge = block_holder->ApproximateMemoryUsage();
-    s = block_cache->Insert(
-        key.AsSlice(), block_holder.get(),
-        BlocklikeTraits<TBlocklike>::GetCacheItemHelper(block_type), charge,
-        nullptr, Cache::Priority::LOW);
+    size_t charge;
+    s = WarmInCache(block_cache, key.AsSlice(), block_contents,
+                    &rep_->create_context, helper, Cache::Priority::LOW,
+                    &charge);
 
     if (s.ok()) {
-      // Release ownership of block_holder.
-      block_holder.release();
       BlockBasedTable::UpdateCacheInsertionMetrics(
           block_type, nullptr /*get_context*/, charge, s.IsOkOverwritten(),
           rep_->ioptions.stats);
@@ -1995,6 +1913,8 @@ Status BlockBasedTableBuilder::Finish() {
     }
   }
 
+  r->props.tail_start_offset = r->offset;
+
   // Write meta blocks, metaindex block and footer in the following order.
   //    1. [meta block: filter]
   //    2. [meta block: index]
@@ -2022,6 +1942,7 @@ Status BlockBasedTableBuilder::Finish() {
   r->SetStatus(r->CopyIOStatus());
   Status ret_status = r->CopyStatus();
   assert(!ret_status.ok() || io_status().ok());
+  r->tail_size = r->offset - r->props.tail_start_offset;
   return ret_status;
 }
 
@@ -2055,6 +1976,8 @@ uint64_t BlockBasedTableBuilder::EstimatedFileSize() const {
   }
 }
 
+uint64_t BlockBasedTableBuilder::GetTailSize() const { return rep_->tail_size; }
+
 bool BlockBasedTableBuilder::NeedCompact() const {
   for (const auto& collector : rep_->table_properties_collectors) {
     if (collector->NeedCompact()) {
