@@ -5,6 +5,8 @@
 //
 
 #include "table/block_based/filter_block_reader_common.h"
+
+#include "block_cache.h"
 #include "monitoring/perf_context_imp.h"
 #include "table/block_based/block_based_table_reader.h"
 #include "table/block_based/parsed_full_filter_block.h"
@@ -16,7 +18,7 @@ Status FilterBlockReaderCommon<TBlocklike>::ReadFilterBlock(
     const BlockBasedTable* table, FilePrefetchBuffer* prefetch_buffer,
     const ReadOptions& read_options, bool use_cache, GetContext* get_context,
     BlockCacheLookupContext* lookup_context,
-    CachableEntry<TBlocklike>* filter_block, BlockType block_type) {
+    CachableEntry<TBlocklike>* filter_block) {
   PERF_TIMER_GUARD(read_filter_block_nanos);
 
   assert(table);
@@ -29,9 +31,9 @@ Status FilterBlockReaderCommon<TBlocklike>::ReadFilterBlock(
   const Status s =
       table->RetrieveBlock(prefetch_buffer, read_options, rep->filter_handle,
                            UncompressionDict::GetEmptyDict(), filter_block,
-                           block_type, get_context, lookup_context,
+                           get_context, lookup_context,
                            /* for_compaction */ false, use_cache,
-                           /* wait_for_cache */ true, /* async_read */ false);
+                           /* async_read */ false);
 
   return s;
 }
@@ -67,8 +69,8 @@ template <typename TBlocklike>
 Status FilterBlockReaderCommon<TBlocklike>::GetOrReadFilterBlock(
     bool no_io, GetContext* get_context,
     BlockCacheLookupContext* lookup_context,
-    CachableEntry<TBlocklike>* filter_block, BlockType block_type,
-    Env::IOPriority rate_limiter_priority) const {
+    CachableEntry<TBlocklike>* filter_block,
+    const ReadOptions& read_options) const {
   assert(filter_block);
 
   if (!filter_block_.IsEmpty()) {
@@ -76,15 +78,14 @@ Status FilterBlockReaderCommon<TBlocklike>::GetOrReadFilterBlock(
     return Status::OK();
   }
 
-  ReadOptions read_options;
-  read_options.rate_limiter_priority = rate_limiter_priority;
+  ReadOptions ro = read_options;
   if (no_io) {
-    read_options.read_tier = kBlockCacheTier;
+    ro.read_tier = kBlockCacheTier;
   }
 
-  return ReadFilterBlock(table_, nullptr /* prefetch_buffer */, read_options,
+  return ReadFilterBlock(table_, nullptr /* prefetch_buffer */, ro,
                          cache_filter_blocks(), get_context, lookup_context,
-                         filter_block, block_type);
+                         filter_block);
 }
 
 template <typename TBlocklike>
@@ -102,8 +103,7 @@ bool FilterBlockReaderCommon<TBlocklike>::RangeMayExist(
     const SliceTransform* prefix_extractor, const Comparator* comparator,
     const Slice* const const_ikey_ptr, bool* filter_checked,
     bool need_upper_bound_check, bool no_io,
-    BlockCacheLookupContext* lookup_context,
-    Env::IOPriority rate_limiter_priority) {
+    BlockCacheLookupContext* lookup_context, const ReadOptions& read_options) {
   if (!prefix_extractor || !prefix_extractor->InDomain(user_key_without_ts)) {
     *filter_checked = false;
     return true;
@@ -117,7 +117,7 @@ bool FilterBlockReaderCommon<TBlocklike>::RangeMayExist(
     *filter_checked = true;
     return PrefixMayMatch(prefix, no_io, const_ikey_ptr,
                           /* get_context */ nullptr, lookup_context,
-                          rate_limiter_priority);
+                          read_options);
   }
 }
 
@@ -157,8 +157,7 @@ bool FilterBlockReaderCommon<TBlocklike>::IsFilterCompatible(
 
 // Explicitly instantiate templates for both "blocklike" types we use.
 // This makes it possible to keep the template definitions in the .cc file.
-template class FilterBlockReaderCommon<BlockContents>;
-template class FilterBlockReaderCommon<Block>;
+template class FilterBlockReaderCommon<Block_kFilterPartitionIndex>;
 template class FilterBlockReaderCommon<ParsedFullFilterBlock>;
 
 }  // namespace ROCKSDB_NAMESPACE
