@@ -7,6 +7,7 @@
 
 #include <map>
 
+#include "block_cache.h"
 #include "index_builder.h"
 #include "rocksdb/filter_policy.h"
 #include "table/block_based/block_based_table_reader.h"
@@ -35,7 +36,8 @@ class MyPartitionedFilterBlockReader : public PartitionedFilterBlockReader {
  public:
   MyPartitionedFilterBlockReader(BlockBasedTable* t,
                                  CachableEntry<Block>&& filter_block)
-      : PartitionedFilterBlockReader(t, std::move(filter_block)) {
+      : PartitionedFilterBlockReader(
+            t, std::move(filter_block.As<Block_kFilterPartitionIndex>())) {
     for (const auto& pair : blooms) {
       const uint64_t offset = pair.first;
       const std::string& bloom = pair.second;
@@ -86,7 +88,8 @@ class PartitionedFilterBlockTest
     int num_keys = sizeof(keys) / sizeof(*keys);
     uint64_t max_key_size = 0;
     for (int i = 1; i < num_keys; i++) {
-      max_key_size = std::max(max_key_size, static_cast<uint64_t>(keys[i].size()));
+      max_key_size =
+          std::max(max_key_size, static_cast<uint64_t>(keys[i].size()));
     }
     uint64_t max_index_size = num_keys * (max_key_size + 8 /*handle*/);
     return max_index_size;
@@ -116,11 +119,11 @@ class PartitionedFilterBlockTest
       PartitionedIndexBuilder* const p_index_builder,
       const SliceTransform* prefix_extractor = nullptr) {
     assert(table_options_.block_size_deviation <= 100);
-    auto partition_size = static_cast<uint32_t>(
-             ((table_options_.metadata_block_size *
-               (100 - table_options_.block_size_deviation)) +
-              99) /
-             100);
+    auto partition_size =
+        static_cast<uint32_t>(((table_options_.metadata_block_size *
+                                (100 - table_options_.block_size_deviation)) +
+                               99) /
+                              100);
     partition_size = std::max(partition_size, static_cast<uint32_t>(1));
     const bool kValueDeltaEncoded = true;
     return new PartitionedFilterBlockBuilder(
@@ -164,7 +167,6 @@ class PartitionedFilterBlockTest
                     PartitionedIndexBuilder* pib, bool empty = false) {
     std::unique_ptr<PartitionedFilterBlockReader> reader(
         NewReader(builder, pib));
-    Env::IOPriority rate_limiter_priority = Env::IO_TOTAL;
     // Querying added keys
     const bool no_io = true;
     for (auto key : keys) {
@@ -173,7 +175,7 @@ class PartitionedFilterBlockTest
       ASSERT_TRUE(reader->KeyMayMatch(key, !no_io, &ikey_slice,
                                       /*get_context=*/nullptr,
                                       /*lookup_context=*/nullptr,
-                                      rate_limiter_priority));
+                                      ReadOptions()));
     }
     {
       // querying a key twice
@@ -182,7 +184,7 @@ class PartitionedFilterBlockTest
       ASSERT_TRUE(reader->KeyMayMatch(keys[0], !no_io, &ikey_slice,
                                       /*get_context=*/nullptr,
                                       /*lookup_context=*/nullptr,
-                                      rate_limiter_priority));
+                                      ReadOptions()));
     }
     // querying missing keys
     for (auto key : missing_keys) {
@@ -192,13 +194,13 @@ class PartitionedFilterBlockTest
         ASSERT_TRUE(reader->KeyMayMatch(key, !no_io, &ikey_slice,
                                         /*get_context=*/nullptr,
                                         /*lookup_context=*/nullptr,
-                                        rate_limiter_priority));
+                                        ReadOptions()));
       } else {
         // assuming a good hash function
         ASSERT_FALSE(reader->KeyMayMatch(key, !no_io, &ikey_slice,
                                          /*get_context=*/nullptr,
                                          /*lookup_context=*/nullptr,
-                                         rate_limiter_priority));
+                                         ReadOptions()));
       }
     }
   }
@@ -351,7 +353,7 @@ TEST_P(PartitionedFilterBlockTest, SamePrefixInMultipleBlocks) {
                                        /*no_io=*/false, &ikey_slice,
                                        /*get_context=*/nullptr,
                                        /*lookup_context=*/nullptr,
-                                       Env::IO_TOTAL));
+                                       ReadOptions()));
   }
   // Non-existent keys but with the same prefix
   const std::string pnonkeys[4] = {"p-key9", "p-key11", "p-key21", "p-key31"};
@@ -362,7 +364,7 @@ TEST_P(PartitionedFilterBlockTest, SamePrefixInMultipleBlocks) {
                                        /*no_io=*/false, &ikey_slice,
                                        /*get_context=*/nullptr,
                                        /*lookup_context=*/nullptr,
-                                       Env::IO_TOTAL));
+                                       ReadOptions()));
   }
 }
 
@@ -393,7 +395,6 @@ TEST_P(PartitionedFilterBlockTest, PrefixInWrongPartitionBug) {
   CutABlock(pib.get(), pkeys[4]);
   std::unique_ptr<PartitionedFilterBlockReader> reader(
       NewReader(builder.get(), pib.get()));
-  Env::IOPriority rate_limiter_priority = Env::IO_TOTAL;
   for (auto key : pkeys) {
     auto prefix = prefix_extractor->Transform(key);
     auto ikey = InternalKey(prefix, 0, ValueType::kTypeValue);
@@ -402,7 +403,7 @@ TEST_P(PartitionedFilterBlockTest, PrefixInWrongPartitionBug) {
                                        /*no_io=*/false, &ikey_slice,
                                        /*get_context=*/nullptr,
                                        /*lookup_context=*/nullptr,
-                                       rate_limiter_priority));
+                                       ReadOptions()));
   }
 }
 
